@@ -9,7 +9,6 @@ from anyscale import snapshot_util, version
 anyscale_dir = os.path.dirname(os.path.abspath(__file__))
 path.append(os.path.join(anyscale_dir, "client"))
 path.append(os.path.join(anyscale_dir, "sdk"))
-ANYSCALE_RAY_DIR = os.path.join(anyscale_dir, "anyscale_ray")
 
 from anyscale import experimental_integrations
 from anyscale.cluster import get_job_submission_client_cluster_info  # noqa: E402
@@ -45,6 +44,5 @@ for attr, _ in inspect.getmembers(ClientBuilder, inspect.isfunction):
 __version__ = version.__version__
 
 ANYSCALE_ENV = os.environ.copy()
-ANYSCALE_ENV["PYTHONPATH"] = ANYSCALE_RAY_DIR + ":" + ANYSCALE_ENV.get("PYTHONPATH", "")
 
 import anyscale.job
