@@ -2,6 +2,12 @@ import copy
 import logging
 from typing import Any, Dict, List
 
+from anyscale.autoscaler.node_provided_cache import NodeProviderCache
+import anyscale.shared_anyscale_utils.conf as shared_anyscale_conf
+from anyscale.utils.import_ray_util import try_import_ray
+
+
+try_import_ray()
 
 # This try block exists because
 # 1. This module is passed to the autoscaler on the head node as an external node provider
@@ -26,9 +32,6 @@ except ModuleNotFoundError:
 
 from ray.autoscaler.tags import TAG_RAY_CLUSTER_NAME, TAG_RAY_NODE_NAME
 
-from anyscale.autoscaler.node_provided_cache import NodeProviderCache
-import anyscale.shared_anyscale_utils.conf as shared_anyscale_conf
-
 
 logger = logging.getLogger(__name__)
 
