@@ -4,40 +4,16 @@ import copy
 import inspect
 import json
 import logging
+import sys
 import time
 from types import ModuleType
 from typing import Any, cast, Dict, List, Optional
 
 from openapi_client.rest import ApiException
-from ray.autoscaler._private.aws.node_provider import to_aws_format
-from ray.autoscaler._private.command_runner import DockerCommandRunner, SSHCommandRunner
-from ray.autoscaler._private.util import hash_launch_conf
-from ray.autoscaler.command_runner import CommandRunnerInterface
-from ray.autoscaler.tags import (
-    STATUS_UP_TO_DATE,
-    TAG_RAY_CLUSTER_NAME,
-    TAG_RAY_NODE_STATUS,
-)
 from urllib3.exceptions import HTTPError  # type: ignore
 import wrapt
 
 from anyscale.authenticate import get_auth_api_client
-
-
-# This try block exists because
-# 1. This module is passed to the autoscaler on the head node as an external node provider
-# 2. Import paths are different for different versions of ray
-try:
-    from ray.autoscaler._private.providers import (  # pylint:disable=private-import
-        _NODE_PROVIDERS,
-    )
-    from ray.autoscaler.node_provider import NodeProvider
-except ModuleNotFoundError:
-    from ray.autoscaler.aws.node_provider import NodeProvider
-    from ray.autoscaler.node_provider import (  # pylint:disable=private-import
-        NODE_PROVIDERS as _NODE_PROVIDERS,
-    )
-
 from anyscale.client.openapi_client.api.default_api import DefaultApi
 from anyscale.client.openapi_client.models.add_instance_pool_member import (
     AddInstancePoolMember,
@@ -60,12 +36,61 @@ from anyscale.client.openapi_client.models.wait_until_stopped_options import (
     WaitUntilStoppedOptions,
 )
 from anyscale.shared_anyscale_utils.util import get_autoscaler_container_path
+from anyscale.utils.import_ray_util import try_import_ray
+
+
+try_import_ray()
+
+from ray.autoscaler._private.aws.node_provider import to_aws_format
+from ray.autoscaler._private.command_runner import DockerCommandRunner, SSHCommandRunner
+from ray.autoscaler._private.util import hash_launch_conf
+from ray.autoscaler.command_runner import CommandRunnerInterface
+from ray.autoscaler.tags import (
+    STATUS_UP_TO_DATE,
+    TAG_RAY_CLUSTER_NAME,
+    TAG_RAY_NODE_STATUS,
+)
+
+
+# This try block exists because
+# 1. This module is passed to the autoscaler on the head node as an external node provider
+# 2. Import paths are different for different versions of ray. ray.autoscaler._private introduced
+#    in Ray 1.0.1
+try:
+    from ray.autoscaler._private.providers import (  # pylint:disable=private-import
+        _NODE_PROVIDERS,
+    )
+    from ray.autoscaler.node_provider import NodeProvider
+except ModuleNotFoundError:
+    from ray.autoscaler.aws.node_provider import NodeProvider
+    from ray.autoscaler.node_provider import (  # pylint:disable=private-import
+        NODE_PROVIDERS as _NODE_PROVIDERS,
+    )
+
+import ray.autoscaler
 
 
 logger = logging.getLogger(__name__)
 RETRY_COUNT = 5
 
 
+def raise_parsed_exception(api_exception: ApiException):
+    # In ray >= 2.1.0, if we throw a NodeLaunchException, the autoscaler can
+    # propagate additional information to the user.
+    error_detail = json.loads(api_exception.body)
+    exception_message = error_detail["error"]["detail"]
+    exc_to_raise: Exception = RuntimeError(exception_message)
+    if hasattr(ray.autoscaler, "node_launch_exception"):
+        headers = api_exception.headers
+        if "category" in headers and "description" in headers:
+            exc_to_raise = ray.autoscaler.node_launch_exception.NodeLaunchException(
+                category=headers["category"],
+                description=headers["description"],
+                src_exc_info=sys.exc_info(),
+            )
+    raise exc_to_raise
+
+
 def wrap_api_client_with_retries(api_client: DefaultApi) -> DefaultApi:
     @wrapt.decorator  # type: ignore
     def retrier(func: Any, instance: Any, args: List[Any], kwargs: Any) -> Any:
@@ -103,7 +128,9 @@ class AnyscaleInstanceManagerNodeProvider(NodeProvider):
         if api_client is None:
             cli_token = self.inner_provider_config["cli_token"]
             self.api_client = wrap_api_client_with_retries(
-                get_auth_api_client(cli_token=cli_token, host=host).api_client
+                get_auth_api_client(
+                    cli_token=cli_token, host=host, raise_structured_exception=True
+                ).api_client
             )
         else:
             self.api_client = api_client
@@ -223,9 +250,7 @@ class AnyscaleInstanceManagerNodeProvider(NodeProvider):
             self.api_client.create_nodes_api_v2_instances_post(create_node_request)
         except ApiException as api_exception:
             # Catch, process, and repropagate remote exceptions.
-            error_detail = json.loads(api_exception.body)
-            exception_message = error_detail["error"]["detail"]
-            raise RuntimeError(exception_message)
+            raise_parsed_exception(api_exception)
 
     def set_node_tags(self, node_id: str, tags: Dict[str, str]) -> None:
         set_node_tags_request = SetNodeTagsOptions(
@@ -408,9 +433,8 @@ class AnyscalePoolingNodeProvider(AnyscaleInstanceManagerNodeProvider):
         # Only handle AWS for now.
         # TODO(xcl): add support for non-aws instances.
         bypass_rapid_start = self._get_from_configs("bypass_rapid_start", None)
-        pool_disabled = self._pool_limit() == 0
-        # Pooling version (with checks).
-        if not (bypass_rapid_start or pool_disabled):
+        # Pooling version (with checks). pool_limit == 0 effectively disables pool.
+        if not (bypass_rapid_start or self._pool_limit() == 0):
             if self._is_aws_provider():
                 for _ in range(count):
                     self._create_single_node_aws(node_config, tags)
