@@ -129,42 +129,20 @@ def _update_file_mounts(
     return cluster_config
 
 
-@_with_safe_cluster_config  # type: ignore
-def _configure_min_max_workers(
-    cluster_config: Dict[str, Any],
-    min_workers: Optional[int] = None,
-    max_workers: Optional[int] = None,
-) -> Dict[str, Any]:
-    if min_workers and "min_workers" in cluster_config:
-        cluster_config["min_workers"] = min_workers
-    if max_workers and "max_workers" in cluster_config:
-        cluster_config["max_workers"] = max_workers
-    if "available_node_types" in cluster_config:
-        for node in cluster_config["available_node_types"]:
-            if min_workers and (
-                "min_workers" in cluster_config["available_node_types"][node]
-            ):
-                cluster_config["available_node_types"][node][
-                    "min_workers"
-                ] = min_workers
-            if max_workers and (
-                "max_workers" in cluster_config["available_node_types"][node]
-            ):
-                cluster_config["available_node_types"][node][
-                    "max_workers"
-                ] = max_workers
-    return cluster_config
-
-
 def configure_for_session(
     session_id: str,
     project_root_dir: Optional[str] = None,
-    api_client: DefaultApi = None,
+    api_client: Optional[DefaultApi] = None,
     disable_project_sync: bool = False,
     _DO_NOT_USE_RAY_UP_ONLY_cluster_config: Optional[  # noqa: N803
         Dict[str, Any]
     ] = None,
 ) -> Dict[str, Any]:
+    """
+    This method constructs the relevant cluster config for a cluster
+    with SSH keys and autoscaler credentials so that CLI commands
+    can run commands on the cluster directly.
+    """
     cluster_config: Dict[str, Any] = {}
     if _DO_NOT_USE_RAY_UP_ONLY_cluster_config is not None:
         cluster_config = _DO_NOT_USE_RAY_UP_ONLY_cluster_config
