@@ -1,4 +1,4 @@
-from typing import Optional
+from typing import List, Optional
 
 import click
 
@@ -116,28 +116,54 @@ def cloud_set_default(
 @click.option(
     "--vpc-peering-target-vpc-id", help="VPC ID of the target VPC.", required=False
 )
+@click.option(
+    "--anyscale-managed",
+    is_flag=True,
+    default=False,
+    hidden=True,
+    help="Let anyscale create all the resources.",
+)
 def setup_cloud(
     provider: str,
     region: str,
     name: str,
     yes: bool,
     gce: bool,
+    anyscale_managed: bool,
     gcp_folder_id: Optional[int],
     vpc_peering_ip_range: Optional[str],
     vpc_peering_target_project_id: Optional[str],
     vpc_peering_target_vpc_id: Optional[str],
 ) -> None:
-    CloudController().setup_cloud(
-        provider=provider,
-        region=region,
-        name=name,
-        yes=yes,
-        gce=gce,
-        folder_id=gcp_folder_id,
-        vpc_peering_ip_range=vpc_peering_ip_range,
-        vpc_peering_target_project_id=vpc_peering_target_project_id,
-        vpc_peering_target_vpc_id=vpc_peering_target_vpc_id,
-    )
+
+    if anyscale_managed:
+        if (
+            vpc_peering_ip_range
+            or vpc_peering_target_vpc_id
+            or vpc_peering_target_project_id
+        ):
+            raise click.ClickException(
+                "VPC peering option is is only available for GCP."
+            )
+        CloudController().setup_managed_cloud(
+            provider=provider,
+            region=region,
+            name=name,
+            yes=yes,
+            folder_id=gcp_folder_id,
+        )
+    else:
+        CloudController().setup_cloud(
+            provider=provider,
+            region=region,
+            name=name,
+            yes=yes,
+            gce=gce,
+            folder_id=gcp_folder_id,
+            vpc_peering_ip_range=vpc_peering_ip_range,
+            vpc_peering_target_project_id=vpc_peering_target_project_id,
+            vpc_peering_target_vpc_id=vpc_peering_target_vpc_id,
+        )
 
 
 @cloud_cli.command(
@@ -267,3 +293,132 @@ def cloud_secrets(
         write_permissions=write_permissions,
         yes=yes,
     )
+
+
+@cloud_cli.command(
+    name="register", help="Register an anyscale cloud with your own resources."
+)
+@click.option(
+    "--provider",
+    help="The cloud provider type.",
+    required=True,
+    type=click.Choice(["aws", "gcp"], case_sensitive=False),
+)
+@click.option(
+    "--region",
+    cls=OptionPromptNull,
+    help="Region to set up the credentials in.",
+    required=True,
+    default_option="provider",
+    default=lambda p: "us-west-2" if p == "aws" else "us-west1",
+    show_default=True,
+)
+@click.option(
+    "--name", "-n", help="Name of the cloud.", required=True,
+)
+@click.option(
+    "--vpc-id", help="The ID of the VPC.", required=False, type=str,
+)
+@click.option(
+    "--subnet-ids",
+    help="Comma separated list of subnet ids.",
+    required=False,
+    type=str,
+)
+@click.option(
+    "--efs-id", help="The EFS ID.", required=False, type=str,
+)
+@click.option(
+    "--anyscale-iam-role-id",
+    help="The Anyscale IAM Role ARN.",
+    required=False,
+    type=str,
+)
+@click.option(
+    "--instance-iam-role-id",
+    help="The instance IAM role ARN.",
+    required=False,
+    type=str,
+)
+@click.option(
+    "--security-group-ids",
+    help="IDs of the security groups.",
+    required=False,
+    type=str,
+)
+@click.option(
+    "--s3-bucket-id", help="S3 bucket ID.", required=False, type=str,
+)
+def register_cloud(
+    provider: str,
+    region: str,
+    name: str,
+    vpc_id: str,
+    subnet_ids: str,
+    efs_id: str,
+    anyscale_iam_role_id: str,
+    instance_iam_role_id: str,
+    security_group_ids: str,
+    s3_bucket_id: str,
+) -> None:
+
+    if provider == "aws":
+        missing_args: List[str] = []
+        for resource in [
+            (vpc_id, "--vpc-id"),
+            (subnet_ids, "--subnet-ids"),
+            (efs_id, "--efs-id"),
+            (anyscale_iam_role_id, "--anyscale-iam-role-id"),
+            (instance_iam_role_id, "--instance-iam-role-id"),
+            (security_group_ids, "--security-group-ids"),
+            (s3_bucket_id, "--s3-bucket-id"),
+        ]:
+            if resource[0] is None:
+                missing_args.append(resource[1])
+
+        if len(missing_args) > 0:
+            raise click.ClickException(f"Please provide a value for {missing_args}")
+
+        CloudController().register_aws_cloud(
+            region=region,
+            name=name,
+            vpc_id=vpc_id,
+            subnet_ids=subnet_ids.split(","),
+            efs_id=efs_id,
+            anyscale_iam_role_id=anyscale_iam_role_id,
+            instance_iam_role_id=instance_iam_role_id,
+            security_group_ids=security_group_ids.split(","),
+            s3_bucket_id=s3_bucket_id,
+        )
+    elif provider == "gcp":
+        # TODO (allenyin): implement this
+        raise click.ClickException(
+            "Registering cloud for GCP is not currently supported"
+        )
+    else:
+        raise click.ClickException(
+            f"Invalid Cloud provider: {provider}. Available providers are [aws, gcp]."
+        )
+
+
+@cloud_cli.command(name="verify", help="Checks the healthiness of a cloud.")
+@click.argument("cloud-name", required=False)
+@click.option("--name", "-n", help="Verify cloud by name.", type=str)
+@click.option(
+    "--cloud-id",
+    "--id",
+    help="Verify cloud by cloud id, alternative to cloud name.",
+    required=False,
+)
+def cloud_verify(
+    cloud_name: Optional[str], name: Optional[str], cloud_id: Optional[str]
+) -> bool:
+    if cloud_name and name and cloud_name != name:
+        raise click.ClickException(
+            "The positional argument CLOUD_NAME and the keyword argument --name "
+            "were both provided. Please only provide one of these two arguments."
+        )
+
+    return CloudController().verify_cloud(
+        cloud_name=cloud_name or name, cloud_id=cloud_id
+    )
