@@ -114,7 +114,9 @@ def cluster_cli() -> None:
 @click.option(
     "--idle-timeout",
     required=False,
-    help="Idle timeout (in minutes), after which the cluster is stopped. Idle "
+    default=None,
+    help="DEPRECATED: Please specify the idle_termination_minutes field in the cluster compute. "
+    "Idle timeout (in minutes), after which the cluster is stopped. Idle "
     "time is defined as the time during which a cluster is not running a user "
     "command and does not have an attached driver. Time spent running Jupyter "
     "commands, or commands run through ssh, is still considered "
