@@ -5,6 +5,7 @@ from typing import List, Optional
 import click
 
 from anyscale.cli_logger import BlockLogger
+from anyscale.client.openapi_client import HaJobStates
 from anyscale.controllers.job_controller import JobController
 import anyscale.job
 from anyscale.util import validate_non_negative_arg
@@ -31,6 +32,15 @@ def job_cli() -> None:
     is_flag=True,
     help="Whether to follow the log of the created job",
 )
+@click.option(
+    "--wait",
+    "-w",
+    required=False,
+    default=False,
+    type=bool,
+    is_flag=True,
+    help="Whether to block this CLI command until the job succeeds (or reaches another terminal state)",
+)
 @click.argument(
     "entrypoint", required=False, nargs=-1,
 )
@@ -40,16 +50,20 @@ def submit(
     name: Optional[str],
     description: Optional[str],
     follow: Optional[bool],
+    wait: Optional[bool],
 ) -> None:
     """ Submit a Production Job
 
     This function accepts 1 argument, a path to a YAML config file that defines this job.
 
 
+    To wait for the Job to complete, use the --wait flag as follows.
+
+    $ anyscale job submit job.yaml --wait
+
     If running from an Anyscale Workspace, this command also support an alternative syntax to specify the entrypoint inline:
-    ```
-    anyscale job submit -- python script.py
-    ```
+
+    $ anyscale job submit -- python script.py
 
     The Cluster Environment and Cluster Config are inferred from the Workspace Configuration if running from a workspace.
 
@@ -65,48 +79,8 @@ def submit(
     )
     if follow:
         job_controller.logs(id, should_follow=follow)
-
-
-@job_cli.command(name="schedule", hidden=True)
-@click.argument("job-config-file", required=True)
-@click.option("--name", "-n", required=False, default=None, help="Name of the job.")
-@click.option("--description", required=False, default=None, help="Description of job.")
-@click.option(
-    "--cron-expression", required=False, default=None, help="Cron expression for job."
-)
-@click.argument(
-    "entrypoint", required=False, nargs=-1,
-)
-def schedule(
-    job_config_file: str,
-    entrypoint: List[str],
-    name: Optional[str],
-    description: Optional[str],
-    cron_expression: Optional[str],
-) -> None:
-    """ Schedule a Production Job
-
-    This function accepts 1 argument, a path to a YAML config file that defines this job.
-
-
-    If running from an Anyscale Workspace, this command also support an alternative syntax to specify the entrypoint inline:
-    ```
-    anyscale job schedule -- python script.py
-    ```
-
-    The Cluster Environment and Cluster Config are inferred from the Workspace Configuration if running from a workspace.
-
-    """
-
-    job_controller = JobController()
-    job_controller.schedule(
-        job_config_file,
-        name,
-        description,
-        entrypoint=entrypoint,
-        is_entrypoint_cmd="--" in sys.argv,
-        cron_expression=cron_expression,
-    )
+    if wait:
+        job_controller.wait(job_id=id)
 
 
 @job_cli.command(name="list", help="Display information about existing jobs.")
@@ -194,6 +168,51 @@ def logs(job_id: Optional[str], name: Optional[str], follow: bool = False,) -> N
     )
 
 
+@job_cli.command(name="wait")
+@click.option("--job-id", "--id", required=False, help="Unique ID of the job.")
+@click.option("--name", "-n", required=False, help="Name of the job.")
+@click.option(
+    "--state",
+    "-s",
+    required=False,
+    default=HaJobStates.SUCCESS,
+    help="The state to wait for this Job to enter",
+)
+@click.option(
+    "--timeout",
+    "-t",
+    required=False,
+    default=None,
+    type=float,
+    help="The timeout in seconds after which this command will exit.",
+)
+def wait(
+    job_id: Optional[str],
+    name: Optional[str],
+    state: str = HaJobStates.SUCCESS,
+    timeout=None,
+) -> None:
+    """Wait for a Job to enter a specific state (default: SUCCESS).
+
+    To specify the Job by name, use the --name flag. To specify the Job by id, use the --job-id flag.
+
+    If the Job reaches the target state, the command will exit successfully.
+
+    If the Job reaches a terminal state other than the target state, the command will exit with an error.
+
+    If the command reaches the timeout, the command will exit with an error.
+    """
+    state = state.upper()
+    job_controller = JobController()
+    job_id = job_controller.wait(
+        job_id=job_id,
+        job_name=name,
+        is_service=False,
+        target_state=state,
+        timeout_secs=timeout,
+    )
+
+
 @job_cli.group("output", help="Interact with Production Job outputs.")
 def output_cli() -> None:
     pass
