@@ -31,8 +31,14 @@ def project_cli() -> None:
 )
 @click.option("--json", help="Format output as JSON.", is_flag=True)
 @click.option(
-    "--any-creator", "-a", help="List projects created by any user.", is_flag=True
+    "--any-creator",
+    "-a",
+    help="[Deprecated] List projects created by any user.",
+    is_flag=True,
+    default=None,
+    hidden=True,
 )
+@click.option("--created-by-me", help="List projects created by me only.", is_flag=True)
 @click.option(
     "--max-items",
     required=False,
@@ -41,39 +47,21 @@ def project_cli() -> None:
     help="Max items to show in list.",
     callback=validate_non_negative_arg,
 )
-def list(name: str, json: bool, any_creator: bool, max_items: int) -> None:
+def list(
+    name: str,
+    json: bool,
+    created_by_me: bool,
+    any_creator: Optional[bool],
+    max_items: int,
+) -> None:
+    if any_creator is not None:
+        log.warning(
+            "`--any-creator` and `-a` flags have been deprecated. "
+            "`anyscale project list` now shows projects created by any user by default. "
+            "If you would like to show projects created by you only, you can pass the --created-by-me flag"
+        )
     project_controller = ProjectController()
-    project_controller.list(name, json, any_creator, max_items)
-
-
-@click.command(
-    name="clone",
-    short_help="DEPRECATED: Clone a project that exists on anyscale, to your local machine.",
-    help="""
-[DEPRECATED]
-
-Clone a project that exists on anyscale, to your local machine.
-This command will create a new folder on your local machine inside of
-the current working directory and download the most recent snapshot.
-
-This is frequently used with anyscale push or anyscale pull to download, make
-changes, then upload those changes to a currently running cluster.""",
-    hidden=True,
-)
-@click.argument("project-name", required=True)
-@click.option(
-    "--owner",
-    help="Username or email of the user who owns the project. Defaults to the current user.",
-    required=False,
-)
-def anyscale_clone(project_name: str, owner: Optional[str]) -> None:
-    project_controller = ProjectController()
-    message = (
-        "Warning: `anyscale clone` has been deprecated and will be removed in the future. "
-        "Please use `anyscale project init --project-id [project_id]` to connect to an existing project."
-    )
-    project_controller.log.warning(message)
-    project_controller.clone(project_name, owner=owner)
+    project_controller.list(name, json, created_by_me, max_items)
 
 
 def _validate_project_name(ctx, param, value) -> str:
