@@ -1,5 +1,5 @@
 import os
-from typing import Any, Optional
+from typing import Any
 
 import click
 
@@ -11,15 +11,11 @@ from anyscale.client.openapi_client.models.execute_shell_command_options import
     ExecuteShellCommandOptions,
 )
 from anyscale.controllers.cluster_controller import ClusterController
-from anyscale.controllers.project_controller import ProjectController
 from anyscale.controllers.workspace_controller import WorkspaceController
-from anyscale.project import (
-    find_project_root,
-    get_proj_name_from_id,
-    write_project_file_to_disk,
-)
+from anyscale.project import find_project_root
 from anyscale.shared_anyscale_utils.utils.byod import BYODInfo
 from anyscale.util import get_endpoint
+from anyscale.workspace import get_workspace_root_or_throw, write_workspace_id_to_disk
 
 
 ANYSCALE_WORKSPACES_SSH_OPTIONS = [
@@ -43,6 +39,7 @@ def workspace_cli() -> None:
 @click.option(
     "--name", "-n", required=True, help="Name of the workspace to create.",
 )
+@click.option("--project-id", required=True)
 @click.option("--cloud-id", required=True)
 @click.option("--cluster-env-build-id", required=False)
 @click.option(
@@ -63,6 +60,7 @@ def workspace_cli() -> None:
 @click.option("--compute-config-id", required=True)
 def create(
     name: str,
+    project_id: str,
     cloud_id: str,
     cluster_env_build_id: str,
     docker: str,
@@ -97,6 +95,7 @@ def create(
     api_client.create_workspace_api_v2_experimental_workspaces_post(
         CreateExperimentalWorkspace(
             name=name,
+            project_id=project_id,
             cloud_id=cloud_id,
             compute_config_id=compute_config_id,
             cluster_environment_build_id=cluster_env_build_id,
@@ -108,18 +107,17 @@ def create(
     name="start", help="Start an existing workspace on Anyscale.", hidden=True
 )
 @click.option(
-    "--name",
-    "-n",
-    required=True,
-    default=None,
-    help="Name of existing workspace to start.",
+    "--name", "-n", required=True, help="Name of existing workspace to start.",
 )
-def start(name: Optional[str],) -> None:
+def start(name: str) -> None:
     cluster_controller = ClusterController()
-    # TODO(ekl) query the workspace cluster name from the DB.
+
+    workspace = get_workspace_from_name(name)
+    cluster_id = workspace.cluster_id
+
     cluster_controller.start(
-        cluster_name=f"workspace-cluster-{name}",
-        cluster_id=None,
+        cluster_name=None,
+        cluster_id=cluster_id,
         cluster_env_name=None,
         docker=None,
         python_version=None,
@@ -138,19 +136,15 @@ def start(name: Optional[str],) -> None:
     name="terminate", help="Terminate a workspace on Anyscale.", hidden=True
 )
 @click.option(
-    "--name",
-    "-n",
-    required=True,
-    default=None,
-    help="Name of existing workspace to terminate.",
+    "--name", "-n", required=True, help="Name of existing workspace to terminate.",
 )
-def terminate(name: Optional[str],) -> None:
+def terminate(name: str) -> None:
     cluster_controller = ClusterController()
+    workspace = get_workspace_from_name(name)
+    cluster_id = workspace.cluster_id
+
     cluster_controller.terminate(
-        cluster_name=f"workspace-cluster-{name}",
-        cluster_id=None,
-        project_id=None,
-        project_name=None,
+        cluster_name=None, cluster_id=cluster_id, project_id=None, project_name=None,
     )
 
 
@@ -163,20 +157,19 @@ def terminate(name: Optional[str],) -> None:
     help="Name of existing workspace to clone.",
 )
 def clone(name: str) -> None:
+    """
+    Clone the workspace to a local dir whose name is the name of the workspace.
+    """
     _check_local()
 
     workspace = get_workspace_from_name(name)
-    project_id = workspace.project_id
-    auth_api_client = get_auth_api_client()
-    dest = get_proj_name_from_id(
-        project_id=project_id, api_client=auth_api_client.api_client
-    )
-    project_controller = ProjectController()
+    dest = workspace.name
+    workspace_controller = WorkspaceController()
     if os.path.exists(dest):
         _exit_error(
             f"Cannot clone workspace: already cloned locally at '{os.path.abspath(dest)}'."
         )
-    project_controller.clone(dest, owner=None)
+    workspace_controller.clone(workspace)
     os.chdir(dest)
     _do_pull(pull_git_state=True)
 
@@ -202,13 +195,13 @@ def activate(name: str) -> None:
         )
     try:
         workspace = get_workspace_from_name(name)
-        id = workspace.project_id
+        workspace_id = workspace.id
     except Exception:
         workpaces_url = get_endpoint("/workspaces")
         raise click.ClickException(
             f"There is no workspace {name} registered with Anyscale. You can view your workspaces here: {workpaces_url}"
         )
-    write_project_file_to_disk(id, root_dir)
+    write_workspace_id_to_disk(workspace_id, root_dir)
 
 
 @workspace_cli.command(name="pull", help="Pull files from a workspace on Anyscale.")
@@ -277,14 +270,14 @@ def run(command: str, web_terminal: bool, as_job: bool, no_push: bool,) -> None:
     # their latest file changes.
     if not no_push:
         _do_push(push_git_state=False)
+    workspace_controller = WorkspaceController()
+    dir_name = workspace_controller.get_workspace_dir_name()
     if web_terminal:
-        workspace_controller = WorkspaceController()
         auth_api_client = get_auth_api_client()
+        cluster_id = workspace_controller.get_activated_workspace().cluster_id
         results = auth_api_client.api_client.execute_shell_command_api_v2_sessions_session_id_execute_shell_command_post(
-            workspace_controller.session.id,
-            ExecuteShellCommandOptions(
-                shell_command=f"cd ~/workspace-project-* && {command}"
-            ),
+            cluster_id,
+            ExecuteShellCommandOptions(shell_command=f"cd ~/{dir_name} && {command}"),
         )
         # TODO(ekl) show the workspace URL here and also block on completion.
         print()
@@ -295,23 +288,24 @@ def run(command: str, web_terminal: bool, as_job: bool, no_push: bool,) -> None:
         print()
     else:
         workspace_controller.run_cmd(
-            ANYSCALE_WORKSPACES_SSH_OPTIONS, f"cd ~/workspace-project-* && {command}",
+            ANYSCALE_WORKSPACES_SSH_OPTIONS, f"cd ~/{dir_name} && {command}",
         )
 
 
 def _do_pull(pull_git_state):
     workspace_controller = WorkspaceController()
+    dir_name = workspace_controller.get_workspace_dir_name()
     # Since workspaces store git objects in an EFS alternates dir, we have to force
     # a repack prior to pulling. Otherwise, the pulled git repo may not be fully
     # functional locally. A repack is expensive, but we assume pulls aren't frequent.
     if pull_git_state:
         workspace_controller.run_cmd(
-            ANYSCALE_WORKSPACES_SSH_OPTIONS,
-            "cd ~/workspace-project-* && git repack -a -d",
+            ANYSCALE_WORKSPACES_SSH_OPTIONS, f"cd ~/{dir_name} && git repack -a -d",
         )
+    workspace_root = get_workspace_root_or_throw()
     workspace_controller.run_rsync(
         ANYSCALE_WORKSPACES_SSH_OPTIONS,
-        workspace_controller.project_definition.root,
+        workspace_root,
         down=True,
         rsync_filters=[".gitignore"],
         rsync_excludes=[".anyscale.yaml", ".git/objects/info/alternates"]
@@ -321,9 +315,10 @@ def _do_pull(pull_git_state):
 
 def _do_push(push_git_state):
     workspace_controller = WorkspaceController()
+    workspace_root = get_workspace_root_or_throw()
     workspace_controller.run_rsync(
         ANYSCALE_WORKSPACES_SSH_OPTIONS,
-        workspace_controller.project_definition.root,
+        workspace_root,
         down=False,
         rsync_filters=[".gitignore"],
         # TODO(ekl) to efficiently push the git state, we need to do this in two
