@@ -1,5 +1,6 @@
 from datetime import datetime
-from typing import Any, Dict, IO, List, Optional
+import pprint
+from typing import Any, Dict, IO, List, Optional, Union
 
 import click
 from click import ClickException
@@ -11,6 +12,7 @@ from yaml.loader import SafeLoader
 from anyscale.cli_logger import BlockLogger
 from anyscale.cloud import get_cloud_id_and_name
 from anyscale.cluster_compute import get_cluster_compute_from_name
+from anyscale.conf import IDLE_TIMEOUT_DEFAULT_MINUTES
 from anyscale.controllers.base_controller import BaseController
 from anyscale.sdk.anyscale_client.models.cluster_computes_query import (
     ClusterComputesQuery,
@@ -26,6 +28,7 @@ from anyscale.sdk.anyscale_client.models.create_cluster_compute_config import (
 )
 from anyscale.sdk.anyscale_client.models.text_query import TextQuery
 from anyscale.util import get_endpoint
+from anyscale.utils.entity_arg_utils import EntityType, IdBasedEntity, NameBasedEntity
 
 
 log = BlockLogger()  # Anyscale CLI Logger
@@ -75,6 +78,17 @@ class CreateClusterComputeConfigModel(BaseModel):
         description="If set to a positive number, Anyscale will terminate the cluster this many minutes after cluster start.",
     )
 
+    idle_termination_minutes: int = Field(
+        IDLE_TIMEOUT_DEFAULT_MINUTES,
+        description=(
+            "If set to a positive number, Anyscale will terminate the cluster this many minutes after the cluster is idle. "
+            "Idle time is defined as the time during which a Cluster is not running a user command. "
+            "Time spent running commands on Jupyter or ssh is still considered 'idle'. "
+            "To disable, set this field to 0."
+        ),
+        ge=0,
+    )
+
     @root_validator
     def fill_cloud_id(cls: Any, values: Any) -> Any:  # noqa: N805
         cloud_id, cloud = (
@@ -138,6 +152,7 @@ class ClusterComputeController(BaseController):
             gcp=cluster_compute_config_model.gcp,
             azure=cluster_compute_config_model.azure,
             maximum_uptime_minutes=cluster_compute_config_model.maximum_uptime_minutes,
+            idle_termination_minutes=cluster_compute_config_model.idle_termination_minutes,
         )
         if name is None:
             name = "cli-config-{}".format(datetime.now().isoformat())
@@ -153,7 +168,9 @@ class ClusterComputeController(BaseController):
         log.info(f"Cluster compute name: {cluster_compute_name}")
 
     def delete(self, cluster_compute_name: Optional[str], id: Optional[str]) -> None:
-        """Deletes the cluster compute with the given name or id.
+        """
+        TODO: shawnp. Remove once archive functionality is enabled on prod.
+        Deletes the cluster compute with the given name or id.
         Exactly one of cluster_compute_name or id must be provided.
         """
 
@@ -180,6 +197,30 @@ class ClusterComputeController(BaseController):
         self.anyscale_api_client.delete_cluster_compute(cluster_compute_id)
         log.info("Cluster compute deleted.")
 
+    def archive(
+        self, compute_config_entity: Union[IdBasedEntity, NameBasedEntity]
+    ) -> None:
+        """
+        Archives the cluster compute with the given name or id.
+        Exactly one of cluster_compute_name or id must be provided.
+        """
+        if compute_config_entity.type is EntityType.ID:
+            compute_config = self.anyscale_api_client.get_compute_template(
+                compute_config_entity.id
+            ).result
+            compute_config_id = compute_config_entity.id
+            compute_config_name = compute_config.name
+        else:
+            compute_config = get_cluster_compute_from_name(
+                compute_config_entity.name, self.api_client
+            )
+            compute_config_id = compute_config.id
+            compute_config_name = compute_config.name
+        self.api_client.archive_compute_template_api_v2_compute_templates_compute_template_id_archive_post(
+            compute_config_id
+        )
+        log.info(f"Successfully archived compute config: {compute_config_name}.")
+
     def list(
         self,
         cluster_compute_name: Optional[str],
@@ -260,8 +301,12 @@ class ClusterComputeController(BaseController):
             cluster_compute_id = get_cluster_compute_from_name(
                 cluster_compute_name, self.api_client
             ).id
-        config = self.api_client.get_compute_template_api_v2_compute_templates_template_id_get(
+        compute_config = self.api_client.get_compute_template_api_v2_compute_templates_template_id_get(
             cluster_compute_id
-        ).result.config
+        ).result
+        model_config = compute_config.config
+
+        formatted_model_config = model_config.to_dict()
+        formatted_model_config["archived_at"] = compute_config.archived_at
         # TODO(nikita): Improve formatting here
-        print(config)
+        pprint.pprint(formatted_model_config)
