@@ -21,14 +21,13 @@ from anyscale.controllers.base_controller import BaseController
 from anyscale.sdk.anyscale_client.models.cluster_compute_config import (
     ClusterComputeConfig,
 )
-from anyscale.utils.cloud_utils import get_default_cloud
-from anyscale.utils.ray_version_checker import (
-    detect_python_minor_version,
-    detect_ray_version,
+from anyscale.util import (
+    DEFAULT_RAY_VERSION,
+    get_ray_and_py_version_for_default_cluster_env,
 )
-
-
-DEFAULT_RAY_VERSION = "1.6.0"
+from anyscale.utils.cloud_utils import get_default_cloud
+from anyscale.utils.import_ray_util import try_import_ray
+from anyscale.utils.ray_version_checker import detect_python_minor_version
 
 
 def run_command_on_all_nodes(command: str, timeout: Optional[float]):
@@ -57,9 +56,7 @@ def run_command_on_all_nodes(command: str, timeout: Optional[float]):
                 # allow time for any logs to propogate before the task exits
                 time.sleep(1)
 
-    # import ray dependencies here to avoid making ray a general dependency for
-    # the CLI
-    import ray
+    ray = try_import_ray()
     from ray.util.placement_group import placement_group
 
     nodes = ray.nodes()
@@ -325,10 +322,10 @@ class ConfigController(BaseController):
         )
 
     def _select_base_image(self, ml: bool, gpu: bool) -> str:
+        ray_version, _ = get_ray_and_py_version_for_default_cluster_env()
         python_version = detect_python_minor_version()
-        ray_version, ray_commit = detect_ray_version()
         self.log.info(
-            f"Detected python version: {python_version}. Detected ray version: {ray_version} and ray commit: {ray_commit}"
+            f"Detected python version: {python_version}. Detected ray version: {ray_version}"
         )
 
         if "dev0" in ray_version:
