@@ -17,6 +17,7 @@ from anyscale.controllers.base_controller import BaseController
 from anyscale.project import get_project_id, get_project_session, load_project_or_throw
 import anyscale.shared_anyscale_utils.conf as shared_anyscale_conf
 from anyscale.shared_anyscale_utils.util import execution_log_name
+from anyscale.utils.import_ray_util import try_import_ray
 
 
 from anyscale.cluster_config import (  # noqa: N812 isort: skip
@@ -25,6 +26,7 @@ from anyscale.cluster_config import (  # noqa: N812 isort: skip
 
 
 def rsync(*args: Any, **kwargs: Any) -> None:
+    try_import_ray()
     from ray.autoscaler.sdk import rsync as ray_rsync
 
     ray_rsync(*args, **kwargs)
@@ -53,6 +55,7 @@ class ExecController(BaseController):
             "to run your script as a job in a cluster."
         )
         self.log.warning(message)
+        try_import_ray()
         import ray.scripts.scripts as autoscaler_scripts
 
         if screen and tmux:
