@@ -1,5 +1,5 @@
 from abc import ABC, abstractmethod
-from datetime import datetime
+from datetime import datetime, timedelta
 import json
 import os
 from pathlib import Path
@@ -19,14 +19,15 @@ import yaml
 from anyscale.cli_logger import LogsLogger
 from anyscale.client.openapi_client import (
     ComputeTemplate,
-    CreateCronJob,
-    CreateProductionJob,
     DecoratedProductionJob,
     HaJobStates,
     JobsLogsQueryInfo,
     ProductionJob,
     ProductionJobConfig,
 )
+from anyscale.client.openapi_client.models.create_internal_production_job import (
+    CreateInternalProductionJob,
+)
 from anyscale.cluster_compute import (
     get_cluster_compute_from_name,
     get_default_cluster_compute,
@@ -45,7 +46,9 @@ from anyscale.util import (
     get_endpoint,
     is_anyscale_workspace,
     poll,
+    populate_dict_with_workspace_config_if_exists,
     PROJECT_NAME_ENV_VAR,
+    validate_job_config_dict,
 )
 from anyscale.utils.runtime_env import override_runtime_env_for_local_working_dir
 
@@ -250,6 +253,9 @@ class JobConfig(BaseModel):
         None,
         description="The id of the project you want to use. If not specified, and no project is inferred from the directory, no project will be used.",
     )
+    workspace_id: Optional[str] = Field(
+        None, description="The id of the workspace that this job is submitted from.",
+    )
     project: Optional[str] = Field(
         None,
         description="The name of the project you want to use. If not specified, and no project is inferred from the directory, no project will be used.",
@@ -284,6 +290,7 @@ class JobConfig(BaseModel):
             values["project_id"] = get_proj_id_from_name(project_name_env_var)
         elif project_name:
             values["project_id"] = get_proj_id_from_name(project_name)
+
         return values
 
     @root_validator
@@ -485,6 +492,11 @@ class LokiLogProvider(LogProvider):
     def query(
         self, start_timestamp_ns: int, end_timestamp_ns: int
     ) -> List[Tuple[int, str]]:
+        if start_timestamp_ns == 0:
+            # 10^9 nanoseconds in a second
+            start_timestamp_ns = int(
+                (datetime.now() - timedelta(days=7)).timestamp() * (10 ** 9)
+            )
         self.params["start"] = start_timestamp_ns
         self.params["end"] = end_timestamp_ns
 
@@ -524,16 +536,21 @@ class JobController(BaseController):
         is_entrypoint_cmd: Optional[bool] = False,
         entrypoint: Optional[List[str]] = None,
     ) -> str:
-
         entrypoint = entrypoint or []
+        workspace_id = os.environ.get("ANYSCALE_EXPERIMENTAL_WORKSPACE_ID", None)
         if is_anyscale_workspace() and is_entrypoint_cmd:
             entrypoint = [job_config_file, *entrypoint]
-            config = self.generate_config_from_entrypoint(entrypoint, name, description)
+            config = self.generate_config_from_entrypoint(
+                entrypoint, name, description, workspace_id
+            )
             id = self.submit_from_config(config)
         elif len(entrypoint) == 0:
             # Assume that job_config_file is a file and submit it.
             config = self.generate_config_from_file(
-                job_config_file, name=name, description=description
+                job_config_file,
+                name=name,
+                description=description,
+                workspace_id=workspace_id,
             )
             id = self.submit_from_config(config)
         elif len(entrypoint) != 0:
@@ -546,21 +563,38 @@ class JobController(BaseController):
         return id
 
     def generate_config_from_entrypoint(
-        self, entrypoint: List[str], name: Optional[str], description: Optional[str],
+        self,
+        entrypoint: List[str],
+        name: Optional[str],
+        description: Optional[str],
+        workspace_id: Optional[str] = None,
     ) -> JobConfig:
         config_dict = {
             "entrypoint": " ".join(entrypoint),
             "name": name,
             "description": description,
+            "workspace_id": workspace_id,
         }
-        return self._populate_job_config(config_dict)
+        config_dict = populate_dict_with_workspace_config_if_exists(
+            config_dict, self.anyscale_api_client
+        )
+        job_config = JobConfig.parse_obj(config_dict)
+        return job_config
 
     def generate_config_from_file(
-        self, job_config_file: str, name: Optional[str], description: Optional[str],
+        self,
+        job_config_file: str,
+        name: Optional[str],
+        description: Optional[str],
+        workspace_id: Optional[str] = None,
     ) -> JobConfig:
-
         config_dict = self._load_config_dict_from_file(job_config_file)
-        job_config = self._populate_job_config(config_dict)
+        config_dict["workspace_id"] = workspace_id
+        validate_job_config_dict(config_dict, self.api_client)
+        config_dict = populate_dict_with_workspace_config_if_exists(
+            config_dict, self.anyscale_api_client
+        )
+        job_config = JobConfig.parse_obj(config_dict)
         if name:
             job_config.name = name
 
@@ -575,7 +609,7 @@ class JobController(BaseController):
             job_config.project_id, self.anyscale_api_client, self.log
         )
         job_config.runtime_env = override_runtime_env_for_local_working_dir(
-            job_config.runtime_env, self.log
+            job_config.runtime_env
         )
 
         config_object = ProductionJobConfig(
@@ -587,10 +621,11 @@ class JobController(BaseController):
         )
 
         job = self.api_client.create_job_api_v2_decorated_ha_jobs_create_post(
-            CreateProductionJob(
+            CreateInternalProductionJob(
                 name=job_config.name or "cli-job-{}".format(datetime.now().isoformat()),
                 description=job_config.description or "Job submitted from CLI",
                 project_id=project_id,
+                workspace_id=job_config.workspace_id,
                 config=config_object,
             )
         ).result
@@ -618,66 +653,6 @@ class JobController(BaseController):
             return f"set to {maximum_uptime_minutes} minutes"
         return "disabled"
 
-    def schedule(
-        self,
-        job_config_file: str,
-        name: Optional[str] = None,
-        description: Optional[str] = None,
-        is_entrypoint_cmd: Optional[bool] = False,
-        entrypoint: Optional[List[str]] = None,
-        cron_expression: Optional[str] = None,
-    ) -> str:
-        entrypoint = entrypoint or []
-        if is_anyscale_workspace() and is_entrypoint_cmd:
-            entrypoint = [job_config_file, *entrypoint]
-            config = self.generate_config_from_entrypoint(entrypoint, name, description)
-            id = self.schedule_from_config(config, cron_expression=cron_expression)
-        elif len(entrypoint) == 0:
-            config = self.generate_config_from_file(
-                job_config_file, name=name, description=description
-            )
-            id = self.schedule_from_config(config, cron_expression=cron_expression)
-        elif len(entrypoint) != 0:
-            msg = (
-                "Within an Anyscale Workspace, `anyscale job schedule` takes either a file, or a command. To submit a command, use `anyscale job schedule -- my command`."
-                if is_anyscale_workspace()
-                else "`anyscale job schedule` takes one argument, a YAML file configuration. Please use `anyscale job schedule my_file`."
-            )
-            raise click.ClickException(msg)
-        return id
-
-    def schedule_from_config(
-        self, job_config: JobConfig, cron_expression: Optional[str] = None
-    ):
-        # If project id is not specified, try to infer it
-        project_id = infer_project_id(
-            job_config.project_id, self.anyscale_api_client, self.log
-        )
-        job_config.runtime_env = override_runtime_env_for_local_working_dir(
-            job_config.runtime_env, self.log
-        )
-
-        config_object = ProductionJobConfig(
-            entrypoint=job_config.entrypoint,
-            runtime_env=job_config.runtime_env,
-            build_id=job_config.build_id,
-            compute_config_id=job_config.compute_config_id,
-            max_retries=job_config.max_retries,
-        )
-
-        job = self.api_client.create_or_update_job_api_v2_experimental_cron_jobs_put(
-            CreateCronJob(
-                name=job_config.name or "cli-job-{}".format(datetime.now().isoformat()),
-                description=job_config.description or "Job submitted from CLI",
-                project_id=project_id,
-                config=config_object,
-                cron_expression=cron_expression,
-            )
-        ).result
-        self.log.info(f"Scheduled Job {job.id} has been created successfully.")
-
-        return job.id
-
     def _load_config_dict_from_file(self, job_config_file: str) -> Dict[str, Any]:
         if not os.path.exists(job_config_file):
             raise click.ClickException(f"Config file {job_config_file} not found.")
@@ -686,26 +661,50 @@ class JobController(BaseController):
             config_dict = yaml.safe_load(f)
         return config_dict
 
-    def _populate_job_config(self, config_dict: Dict[str, Any]) -> JobConfig:
+    def wait(
+        self,
+        job_name: Optional[str] = None,
+        job_id: Optional[str] = None,
+        is_service: bool = False,
+        target_state: str = HaJobStates.SUCCESS,
+        timeout_secs=None,
+    ):
+        if target_state not in HaJobStates.allowable_values:
+            raise click.ClickException(
+                f"`{target_state}` is not a valid Job state. Allowed states are {HaJobStates.allowable_values}"
+            )
+        job_id_or_name = job_name or job_id
+        job_id = cast(
+            str,
+            self._resolve_job_object(
+                job_id, job_name, entity_type="service" if is_service else "job"
+            ).id,
+        )
+        with self.log.spinner(f"Waiting for Job `{job_id_or_name}`...") as spinner:
+            for _ in poll(timeout_secs=timeout_secs):
+                state = self._get_job_state(job_id)
+                spinner.text = f"Job `{job_id_or_name}` is in state `{state}`. Waiting to reach state `{target_state}`."
 
-        if "ANYSCALE_EXPERIMENTAL_WORKSPACE_ID" in os.environ:
-            if "ANYSCALE_SESSION_ID" in os.environ:
-                cluster = self.anyscale_api_client.get_cluster(
-                    os.environ["ANYSCALE_SESSION_ID"]
-                ).result
-                # If the job configs are not specified, infer them from the workspace:
-                if "build_id" not in config_dict and "cluster_env" not in config_dict:
-                    config_dict["build_id"] = cluster.cluster_environment_build_id
-                if "project_id" not in config_dict:
-                    config_dict["project_id"] = cluster.project_id
-                if (
-                    "compute_config" not in config_dict
-                    and "compute_config_id" not in config_dict
-                ):
-                    config_dict["compute_config_id"] = cluster.cluster_compute_id
+                if state == target_state:
+                    spinner.succeed(
+                        f"Job `{job_id_or_name}` reached state `{target_state}`."
+                    )
+                    return job_id
 
-        job_config = JobConfig.parse_obj(config_dict)
-        return job_config
+                if state in terminal_state:
+                    msg = f"Job `{job_id_or_name}` reached terminal state `{state}`, and will not reach `{target_state}`."
+                    spinner.fail(msg)
+                    raise click.ClickException(msg)
+
+            msg = f"Timed out after waiting for {timeout_secs} seconds. The current state is {state}."
+            spinner.fail(msg)
+            raise click.ClickException(msg)
+
+    def _get_job_state(self, job_id: str):
+        job = self.api_client.get_job_api_v2_decorated_ha_jobs_production_job_id_get(
+            job_id
+        ).result
+        return job.state.current_state
 
     def list(
         self,
