@@ -21,6 +21,7 @@ from anyscale.project import (
     load_project_or_throw,
 )
 from anyscale.util import get_endpoint, humanize_timestamp
+from anyscale.utils.import_ray_util import try_import_ray
 
 
 class ListController(BaseController):
@@ -279,6 +280,7 @@ class ListController(BaseController):
 
     # TODO (jbai): Reorganize to other files
     def _get_head_and_worker_ips(self, session_name: str) -> Tuple[str, List[str]]:
+        try_import_ray()
         from ray.autoscaler.sdk import get_head_node_ip, get_worker_node_ips
 
         cluster_config = get_cluster_config(session_name, self.api_client)
