@@ -85,7 +85,7 @@ class ServiceController(BaseController):
         entrypoint: List[str],
         name: Optional[str],
         description: Optional[str],
-        healthcheck_url: Optional[str] = None,
+        healthcheck_url: Optional[str] = "/healthcheck",
     ) -> ServiceConfig:
         config_dict = {
             "entrypoint": " ".join(entrypoint),
@@ -126,7 +126,7 @@ class ServiceController(BaseController):
         )
 
         service_config.runtime_env = override_runtime_env_for_local_working_dir(
-            service_config.runtime_env, self.log
+            service_config.runtime_env
         )
 
         config_object = ProductionJobConfig(
