@@ -5,9 +5,11 @@ import traceback
 from typing import Any, Dict, Optional, Tuple
 
 from anyscale.authenticate import get_auth_api_client
+from anyscale.util import str_data_size
 
 
 logger = logging.getLogger("Job Outputs")
+
 ANYSCALE_HA_JOB_ID = "ANYSCALE_HA_JOB_ID"
 ANYSCALE_RAY_JOB_SUBMISSION_ID = "ANYSCALE_RAY_JOB_SUBMISSION_ID"
 
@@ -15,14 +17,14 @@ ANYSCALE_RAY_JOB_SUBMISSION_ID = "ANYSCALE_RAY_JOB_SUBMISSION_ID"
 def output(result: Dict[str, Any], *, _api_client=None, _fail_on_error=False):
     """Submit the output for an Anyscale Production Job
 
-    This function will only record outputs when called from within an Anyscale Production Job, and will otherwise print the output.
+    Outputs can only be recorded from within an Anyscale Production Job.
 
     This function should be called from the driver of your ray script.
 
-    Example usage for this function can be found in the Anyscale Documentation.
+    Example usage for this function can be found in the Anyscale documentation.
 
     Args:
-        result (Dict[str, Any]): A JSON serializable dictionary representing your output data. Size limit is 1MB.
+        result (Dict[str, Any]): A JSON serializable dictionary representing your output data. Size limit is 1MiB.
 
     Raises:
         e: if the internal flag _fail_on_error is set, this function will raise any exceptions from the API
@@ -35,18 +37,27 @@ def output(result: Dict[str, Any], *, _api_client=None, _fail_on_error=False):
             return
 
         try:
-            pretty_data = json.dumps(result, indent=2)
-            print(f"Submitting structured output to Anyscale:\n{pretty_data}")
+            serialized_result = json.dumps(result, indent=2)
         except Exception:
             traceback.print_exc()
             logger.warn(
-                "Unable to serialize the structured output data to JSON, your structured output will not be recorded."
+                "Failed to serialize the job output to JSON. Output will not be recorded."
             )
             return
 
-        ids = _get_ha_job_id_from_environment()
+        if str_data_size(serialized_result) > 1024 ** 2:
+            logger.warn(
+                "Job output exceeds maximum size of 1MiB. Output will not be recorded."
+            )
+            return
 
-        if not ids:
+        # Check if we're in a production job last so we still go through the above checks
+        # to aid users with debugging.
+        ids = _get_ha_job_id_from_environment()
+        if ids is None:
+            logger.warn(
+                "Job outputs can only be recorded from Production Jobs. Output will be ignored."
+            )
             return
 
         ha_job_id, ray_job_submission_id = ids
@@ -85,14 +96,13 @@ def _get_ha_job_id_from_environment() -> Optional[Tuple[str, str]]:
     ha_job_id = os.getenv(ANYSCALE_HA_JOB_ID)
     ray_job_submission_id = os.getenv(ANYSCALE_RAY_JOB_SUBMISSION_ID)
 
+    if not ha_job_id:
+        logger.debug(f"{ANYSCALE_HA_JOB_ID} env var not set.")
+
+    if not ray_job_submission_id:
+        logger.debug(f"{ANYSCALE_RAY_JOB_SUBMISSION_ID} env var not set.")
+
     if ha_job_id and ray_job_submission_id:
         return ha_job_id, ray_job_submission_id
 
-    if not ha_job_id:
-        logger.warn(f"Expected environment variable '{ANYSCALE_HA_JOB_ID}' to be set.")
-    if not ray_job_submission_id:
-        logger.warn(
-            f"Expected environment variable '{ANYSCALE_RAY_JOB_SUBMISSION_ID}' to be set."
-        )
-    logger.warn("Structured outputs are only recorded from Anyscale Production Jobs.")
     return None
