@@ -1,7 +1,6 @@
 import json
 import logging
 import os
-import sys
 from typing import Any
 
 import click
@@ -14,19 +13,16 @@ from anyscale.commands.cluster_commands import cluster_cli
 from anyscale.commands.cluster_compute_commands import cluster_compute_cli
 from anyscale.commands.cluster_env_commands import cluster_env_cli
 from anyscale.commands.config_commands import config_cli
-from anyscale.commands.download_logs_commands import log_cli
 from anyscale.commands.exec_commands import anyscale_exec
 from anyscale.commands.experimental_integrations_commands import (
     experimental_integrations_cli,
 )
 from anyscale.commands.job_commands import job_cli
 from anyscale.commands.list_commands import list_cli
+from anyscale.commands.logs_commands import log_cli
 from anyscale.commands.migrate_commands import migrate_cli
-from anyscale.commands.project_commands import (
-    anyscale_clone,
-    anyscale_init,
-    project_cli,
-)
+from anyscale.commands.project_commands import anyscale_init, project_cli
+from anyscale.commands.schedule_commands import schedule_cli
 from anyscale.commands.service_commands import service_cli
 from anyscale.commands.session_commands import (
     anyscale_pull,
@@ -34,7 +30,6 @@ from anyscale.commands.session_commands import (
     anyscale_ssh,
     anyscale_start,
     anyscale_stop,
-    anyscale_up,
 )
 from anyscale.commands.session_commands_hidden import session_cli
 from anyscale.commands.workspace_commands import workspace_cli
@@ -62,6 +57,7 @@ class AliasedGroup(click.Group):
 
 
 @click.group(
+    "anyscale",
     invoke_without_command=True,
     no_args_is_help=True,
     cls=AliasedGroup,
@@ -84,9 +80,6 @@ class AliasedGroup(click.Group):
 )
 @click.pass_context
 def cli(ctx: Any, version_flag: bool, show_json: bool) -> None:
-    # Use anyscale vendored Ray for the CLI:
-    sys.path.insert(0, anyscale.ANYSCALE_RAY_DIR)
-
     if version_flag:
         ctx.invoke(version_cli, show_json=show_json)
 
@@ -119,6 +112,7 @@ cli.add_command(version_cli)
 cli.add_command(list_cli)
 cli.add_command(cluster_env_cli)
 cli.add_command(job_cli)
+cli.add_command(schedule_cli)
 cli.add_command(service_cli)
 cli.add_command(cluster_cli)
 cli.add_command(workspace_cli)
@@ -126,9 +120,7 @@ cli.add_command(experimental_integrations_cli)
 cli.add_command(auth_cli)
 
 cli.add_command(anyscale_init)
-cli.add_command(anyscale_up)
 cli.add_command(anyscale_stop)
-cli.add_command(anyscale_clone)
 cli.add_command(anyscale_ssh)
 cli.add_command(anyscale_start)
 cli.add_command(anyscale_exec)
@@ -142,7 +134,12 @@ cli.add_command(anyscale_api)
 
 cli.add_command(log_cli)
 
-ALIASES = {"h": anyscale_help}
+ALIASES = {
+    "h": anyscale_help,
+    "schedules": schedule_cli,
+    "jobs": job_cli,
+    "services": service_cli,
+}
 
 
 def main() -> Any:
