@@ -1,6 +1,6 @@
 import sys
 from time import sleep, time
-from typing import Any, Dict, Optional
+from typing import Any, Dict, Optional, Union
 
 from anyscale.api import configure_tcp_keepalive
 from anyscale.sdk.anyscale_client.models.cluster_compute_config import ClusterComputeConfig
@@ -12,6 +12,9 @@ from anyscale.version import __version__ as version
 from anyscale.sdk.anyscale_client.models.create_cluster_environment import (
     CreateClusterEnvironment,
 )
+from anyscale.sdk.anyscale_client.models.create_byod_cluster_environment import (
+    CreateBYODClusterEnvironment,
+)
 from anyscale.sdk.anyscale_client.models.cluster import Cluster
 from anyscale.sdk.anyscale_client.models.cluster_environment_build import (
     ClusterEnvironmentBuild,
@@ -21,6 +24,7 @@ from anyscale.cli_logger import BlockLogger
 from anyscale.util import get_endpoint
 from anyscale.authenticate import AuthenticationBlock
 from anyscale_client.models import ClusterEnvironmentBuildStatus
+from anyscale.util import get_ray_and_py_version_for_default_cluster_env
 
 def _upload_and_rewrite_working_dir_in_create_production_job(create_production_job):
     if create_production_job.config is not None:
@@ -29,6 +33,20 @@ def _upload_and_rewrite_working_dir_in_create_production_job(create_production_j
             new_runtime_env = upload_and_rewrite_working_dir(runtime_env)
             create_production_job.config["runtime_env"] = new_runtime_env
 
+
+def _is_create_byod_cluster_environment(obj: Any) -> bool:
+    """
+    Returns True if the object's config_json has "docker_image" set.
+    The object can be either a dictionary or an object with "config_json"
+    as an attribute.
+    """
+    config_json = None
+    if isinstance(obj, dict):
+        config_json = obj.get("config_json")
+    else:
+        config_json = getattr(obj, "config_json", None)
+    return isinstance(config_json, dict) and "docker_image" in config_json
+
 # Build states that are in progress
 IN_PROGRESS_BUILD_STATES = [ClusterEnvironmentBuildStatus.IN_PROGRESS, ClusterEnvironmentBuildStatus.PENDING]
 
@@ -59,7 +77,7 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
 
     def build_cluster_environment(
         self,
-        create_cluster_environment: CreateClusterEnvironment,
+        create_cluster_environment: Union[CreateClusterEnvironment, CreateBYODClusterEnvironment],
         poll_rate_seconds: int = 15,
         timeout_seconds: Optional[int] = None,
         log_output: bool = False,
@@ -94,9 +112,14 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
             self.log.info(
                 f"Creating new cluster environment {create_cluster_environment.name}"
             )
-            cluster_environment = self.create_cluster_environment(
-                create_cluster_environment
-            ).result
+            if _is_create_byod_cluster_environment(create_cluster_environment):
+                cluster_environment = self.create_byod_cluster_environment(
+                    create_cluster_environment
+                ).result
+            else:
+                cluster_environment = self.create_cluster_environment(
+                    create_cluster_environment
+                ).result
             build = self.list_cluster_environment_builds(
                 cluster_environment.id
             ).results[0]
@@ -106,12 +129,20 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
                 f"Building new revision of cluster environment {create_cluster_environment.name}"
             )
             cluster_environment = cluster_environments[0]
-            build = self.create_cluster_environment_build(
-                {
-                    "cluster_environment_id": cluster_environment.id,
-                    "config_json": create_cluster_environment.config_json,
-                }
-            ).result
+            if _is_create_byod_cluster_environment(create_cluster_environment):
+                build = self.create_byod_cluster_environment_build(
+                    {
+                        "cluster_environment_id": cluster_environment.id,
+                        "config_json": create_cluster_environment.config_json,
+                    }
+                ).result
+            else:
+                build = self.create_cluster_environment_build(
+                    {
+                        "cluster_environment_id": cluster_environment.id,
+                        "config_json": create_cluster_environment.config_json,
+                    }
+                ).result
             build_operation_id = build.id
 
         return self.wait_for_cluster_environment_build_operation(
@@ -194,7 +225,7 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
             cluster_environment_build_id - Cluster Environment Build to start this Cluster with
                                            If none, uses a default cluster_environment_build
             cluster_compute_id - Cluster Compute to start this Cluster with
-                                 If none, it checks if `cluster_compute_config` is specified. 
+                                 If none, it checks if `cluster_compute_config` is specified.
                                  If `cluster_compute_config` is none, it uses a default cluster_compute.
             poll_rate_seconds - seconds to wait when polling Cluster operation status; defaults to 15
             timeout_seconds - maximum number of seconds to wait for Cluster operation to complete before timing out; defaults to no timeout
@@ -223,21 +254,7 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
         else:
             if not cluster_environment_build_id:
                 # Use default cluster environment build when starting cluster without specifying an id
-                py_version = "".join(str(x) for x in sys.version_info[0:2])
-                if py_version not in ["36", "37", "38"]:
-                    raise ValueError(
-                        "No default cluster env for python version {}. Please use a version of python between 3.6 and 3.8.".format(
-                            py_version
-                        )
-                    )
-                import ray
-
-                ray_version = ray.__version__
-                if "dev0" in ray_version:
-                    raise ValueError(
-                        f"Your locally installed Ray version is {ray_version}. "
-                        "There is no default cluster environments for nightly versions of Ray."
-                    )
+                ray_version, py_version = get_ray_and_py_version_for_default_cluster_env()
                 cluster_environment_build_id = self.get_default_cluster_environment_build(
                     f"py{py_version}", ray_version
                 ).result.id
@@ -293,7 +310,7 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
             cluster_name - Name of the Cluster
             create_cluster_environment - CreateClusterEnvironment object
             cluster_compute_id - Cluster Compute to start this Cluster with
-                                 If none, it checks if `cluster_compute_config` is specified. 
+                                 If none, it checks if `cluster_compute_config` is specified.
                                  If `cluster_compute_config` is  none, it uses a default cluster_compute.
             poll_rate_seconds - seconds to wait when polling for operations; defaults to 15
             timeout_seconds - maximum number of seconds to wait for each operation to complete before timing out; defaults to no timeout
