@@ -6,7 +6,9 @@ import shutil
 import subprocess
 import tempfile
 import time
-from typing import Any, Dict, Optional
+from typing import Any, Dict, Optional, Tuple
+
+from anyscale.utils.import_ray_util import try_import_ray
 
 
 logger = logging.getLogger(__name__)
@@ -17,6 +19,7 @@ WORKING_DIR = os.environ.get("ANYSCALE_WORKING_DIR", "/home/ray")
 # These files are saved per workspace.
 CONFIGS_TO_SAVE = [
     ".bash_history",
+    ".zsh_history",
 ]
 # These files are saved per user.
 CONFIGS_TO_REPLICATE = [
@@ -27,10 +30,17 @@ CONFIGS_TO_REPLICATE = [
 
 # Experimental configs propagated from the cluster config service.
 EFS_IP = os.environ.get("ANYSCALE_EXPERIMENTAL_EFS_IP", "")
+EFS_ROOT = os.environ.get("ANYSCALE_EXPERIMENTAL_EFS_ROOT", "")
+EFS_TYPE = os.environ.get("ANYSCALE_EXPERIMENTAL_EFS_TYPE", "")
 # Override with container-defined IP.
 CUSTOM_EFS_IP = os.environ.get("CUSTOM_EFS_IP", "")
+CUSTOM_EFS_ROOT = os.environ.get("CUSTOM_EFS_ROOT", "")
 if CUSTOM_EFS_IP:
     EFS_IP = CUSTOM_EFS_IP
+
+if CUSTOM_EFS_ROOT:
+    EFS_ROOT = CUSTOM_EFS_ROOT
+
 WORKSPACE_ID = os.environ.get("ANYSCALE_EXPERIMENTAL_WORKSPACE_ID", "")
 USERNAME = os.environ.get("ANYSCALE_EXPERIMENTAL_USERNAME", "unknown_user")
 BASE_SNAPSHOT = os.environ.get("ANYSCALE_EXPERIMENTAL_BASE_SNAPSHOT") or None
@@ -44,6 +54,10 @@ EFS_CREDS_DIR = os.environ.get("EFS_CREDS_DIR", "/efs/generated_credentials")
 RAY_ML_DEV = bool(os.environ.get("RAY_ML_DEV"))
 AUTOGC = bool(os.environ.get("AUTOGC_SNAPSHOTS", True))
 
+# TODO(ekl) enable this flag by default. This causes workspaces to use the $CWD
+# as the runtime env working dir instead of always the workspace root.
+RELATIVE_WORKING_DIR = bool(os.environ.get("RELATIVE_WORKING_DIR", False))
+
 
 def optimize_git_repo(directory: str, shared_repo: str) -> None:
     """Optimize the space usage of a git repo by syncing objects to a shared repo.
@@ -107,9 +121,13 @@ def create_snapshot_zip(directory: str, auto: bool) -> str:
     if auto:
         prefix += "auto_"
     target = tempfile.mktemp(suffix=".zip", prefix=prefix)
+    is_git_workspace = os.path.exists(os.path.join(WORKING_DIR, ".git"))
+    is_within_workspace = os.path.abspath(directory).startswith(
+        os.path.abspath(WORKING_DIR)
+    )
     try:
         os.chdir(directory)
-        if os.path.exists(".git"):
+        if is_git_workspace and is_within_workspace:
             subprocess.check_call(  # noqa
                 "(git ls-files -co --exclude-standard || true; find .git || true) | "
                 f"zip --symlinks -@ -0 -q {target}",
@@ -220,9 +238,9 @@ def do_snapshot(auto: bool = False):
         zip + ".md5", os.path.join(snapshot_dir, os.path.basename(zip) + ".md5")
     )
     for config in CONFIGS_TO_SAVE:
-        shutil.copy(
-            os.path.join("/home/ray", config), os.path.join(workspace_dir, config),
-        )
+        source = os.path.join("/home/ray", config)
+        if os.path.exists(source):
+            shutil.copy(source, os.path.join(workspace_dir, config))
     if AUTOGC:
         gc_snapshots()
 
@@ -332,6 +350,14 @@ def restore_latest():
             shutil.copy(saved, os.path.join("/home/ray", config))
 
 
+# Copied from https://stackoverflow.com/a/50864227
+def _split_s3_path(s3_path: str) -> Tuple[str, str]:
+    path_parts = s3_path.replace("s3://", "").split("/")
+    bucket = path_parts.pop(0)
+    key = "/".join(path_parts)
+    return bucket, key
+
+
 def checkpoint_job(job_id, runtime_env: Dict[str, Any]) -> Dict[str, Any]:
     """Checkpoint the runtime environment and working directory of a job.
 
@@ -348,9 +374,19 @@ def checkpoint_job(job_id, runtime_env: Dict[str, Any]) -> Dict[str, Any]:
         # We're a job, also save a replica of the zip in EFS so the job can be cloned
         # as a workspace at a later time.
         working_dir = os.path.join(dest_dir, "working_dir.zip")
-        import urllib.request
 
-        urllib.request.urlretrieve(runtime_env["working_dir"], working_dir)
+        # We need a custom path for s3 urls because urllib.request.urlretrieve doesn't work for s3 urls
+        if runtime_env["working_dir"].startswith("s3://"):
+            import boto3
+
+            bucket, key = _split_s3_path(runtime_env["working_dir"])
+            # boto3 client initialized inside clusters already have AWS credentials set
+            s3_client = boto3.client("s3")
+            s3_client.download_file(bucket, key, working_dir)
+        else:
+            import urllib.request
+
+            urllib.request.urlretrieve(runtime_env["working_dir"], working_dir)
         runtime_env["working_dir"] = working_dir
     # Save the runtime_env to be used later.
     with open(os.path.join(dest_dir, "runtime_env.json"), "w") as f:
@@ -369,7 +405,7 @@ def setup_ml_dev(runtime_env):
     """
     if not runtime_env:
         runtime_env = {}
-    import ray
+    ray = try_import_ray()
 
     sys_ray_module = os.path.dirname(ray.__file__)
     local_ray_module = os.path.join(WORKING_DIR, "python/ray")
@@ -419,20 +455,30 @@ def env_hook(runtime_env) -> Dict[str, Any]:
 
     This should be set as `RAY_RUNTIME_ENV_HOOK=anyscale.snapshot_util.env_hook`.
     """
-    if "ANYSCALE_EXPERIMENTAL_JOB_ID" in os.environ:
-        return checkpoint_job(os.environ["ANYSCALE_EXPERIMENTAL_JOB_ID"], runtime_env)
     if not runtime_env:
         runtime_env = {}
+
+    # If EFS fails to mount, we don't update the working_dir to EFS
+    if not os.path.ismount("/efs"):
+        return runtime_env
+
+    if "ANYSCALE_EXPERIMENTAL_INITIAL_JOB_ID" in os.environ:
+        return checkpoint_job(
+            os.environ["ANYSCALE_EXPERIMENTAL_INITIAL_JOB_ID"], runtime_env
+        )
+
     # Only fill in the working_dir if it's non-empty (to avoid empty zip errors).
     if not runtime_env.get("working_dir") and os.listdir(WORKING_DIR):
         optimize_git_repo(WORKING_DIR, EFS_OBJECTS_DIR)
-        zipfile = get_or_create_snapshot_zip(WORKING_DIR, auto=False)
+        if RELATIVE_WORKING_DIR:
+            zipfile = create_snapshot_zip(".", auto=False)
+        else:
+            zipfile = create_snapshot_zip(WORKING_DIR, auto=False)
         # Move the zip file to a consistent path so that we don't leak zip files as
         # jobs are run over time. This isn't thread safe: we assume one job at a time.
         # Note that the zip file isn't needed after the job starts successfully.
         final_path = "/tmp/ray_latest_runtime_env.zip"
         shutil.move(zipfile, final_path)
-        os.unlink(zipfile + ".md5")
         runtime_env["working_dir"] = final_path
     if RAY_ML_DEV:
         runtime_env = setup_ml_dev(runtime_env)
@@ -543,14 +589,15 @@ def put_env_var(key: str, value: str) -> None:
 
 
 def setup_nfs():
-    """Setup EFS mounts in the container."""
+    """Setup EFS or FileStore mounts in the container."""
+    EFS_MOUNT = f"sudo mount -t nfs4 -o nfsvers=4.1,rsize=1048576,wsize=1048576,hard,timeo=600,retrans=2,noresvport {EFS_IP}:/ /efs"
+    FILE_STORE_MOUNT = f"sudo mount -o rsize=1048576,wsize=1048576,hard,timeo=600,retrans=2,nolock {EFS_IP}:/{EFS_ROOT} /efs"
     COMMANDS = [
         # TODO: move dep install to base image
         "sudo apt-get update",
         "sudo apt-get install -y nfs-common zip unzip awscli",
         "sudo mkdir -p /efs",
-        "sudo mount -t nfs4 -o nfsvers=4.1,rsize=1048576,wsize=1048576,hard,"
-        f"timeo=600,retrans=2,noresvport {EFS_IP}:/ /efs",
+        EFS_MOUNT if EFS_TYPE == "EFS" else FILE_STORE_MOUNT,
         "sudo mkdir -p /efs",
         "sudo chown ray /efs",
         f"mkdir -p /efs/workspaces/{WORKSPACE_ID}/cluster_storage",
