@@ -1,7 +1,7 @@
 from contextlib import contextmanager
 from copy import deepcopy
 import datetime
-import json
+import ipaddress
 import logging
 import os
 import random
@@ -10,22 +10,42 @@ import subprocess
 import sys
 import threading
 import time
-from typing import Any, Callable, cast, Dict, Generator, Iterator, List, Optional, Tuple
+from typing import (
+    Any,
+    Callable,
+    cast,
+    Dict,
+    Generator,
+    Iterator,
+    List,
+    Optional,
+    Tuple,
+    Union,
+)
 from urllib.parse import quote as quote_sanitize, urlencode, urljoin
 import webbrowser
 
-from botocore.exceptions import NoRegionError
+import boto3
+from botocore.config import Config
+from botocore.exceptions import ClientError, NoRegionError
 import click
+from packaging import version
 import requests
 from requests import Response
 import yaml
 
 from anyscale.authenticate import get_auth_api_client
 from anyscale.cli_logger import BlockLogger
-from anyscale.client.openapi_client.api.default_api import DefaultApi
+from anyscale.client.openapi_client.api.default_api import DefaultApi as ProductApi
+from anyscale.client.openapi_client.models.decorated_compute_template import (
+    DecoratedComputeTemplate,
+)
 from anyscale.client.openapi_client.models.user_info import UserInfo
-from anyscale.client.openapi_client.rest import ApiException
+from anyscale.cluster_compute import get_cluster_compute_from_name
 import anyscale.conf
+from anyscale.conf import MINIMUM_RAY_VERSION
+from anyscale.sdk.anyscale_client.api.default_api import DefaultApi as BaseApi
+from anyscale.sdk.anyscale_client.models.compute_template import ComputeTemplate
 import anyscale.shared_anyscale_utils.conf as shared_anyscale_conf
 from anyscale.shared_anyscale_utils.util import get_container_name
 
@@ -39,10 +59,9 @@ VALID_PYTHON_VERSIONS = ["py36", "py37", "py38", "py39"]
 
 log = BlockLogger()  # Anyscale CLI Logger
 
-# Cached 07-21-2021
-_CACHED_GCP_REGIONS = json.loads(
-    open(os.path.join(os.path.dirname(__file__), "cached_gcp_regions.json")).read()
-)["regions"]
+VPC_CIDR_RANGE = "10.0.0.0/16"
+
+DEFAULT_RAY_VERSION = "1.7.0"
 
 
 def confirm(msg: str, yes: bool) -> Any:
@@ -59,6 +78,9 @@ class AnyscaleEndpointFormatter:
     def get_job_endpoint(self, job_id: str) -> str:
         return self.get_endpoint(f"/jobs/{job_id}")
 
+    def get_schedule_endpoint(self, schedule_id: str) -> str:
+        return self.get_endpoint(f"/scheduled-jobs/{schedule_id}")
+
 
 def get_endpoint(endpoint: str, host: Optional[str] = None) -> str:
     return str(urljoin(host or shared_anyscale_conf.ANYSCALE_HOST, endpoint))
@@ -168,9 +190,6 @@ def get_requirements(requirements_path: str) -> str:
 
 
 def _resource(name: str, region: str) -> Any:
-    import boto3
-    from botocore.config import Config
-
     boto_config = Config(retries={"max_attempts": BOTO_MAX_RETRIES})
     return boto3.resource(name, region, config=boto_config)
 
@@ -180,38 +199,32 @@ def _client(name: str, region: str) -> Any:
 
 
 def _get_role(role_name: str, region: str) -> Any:
-    import botocore
-
     iam = _resource("iam", region)
     role = iam.Role(role_name)
     try:
         role.load()
         return role
-    except botocore.exceptions.ClientError as exc:
+    except ClientError as exc:
         if exc.response.get("Error", {}).get("Code") == "NoSuchEntity":
             return None
         else:
             raise exc
 
 
-def _get_user(user_name: str, region: str) -> Any:
-    import botocore
-
-    iam = _resource("iam", region)
-    user = iam.User(user_name)
+def _get_subnet(subnet_arn: str, region: str) -> Any:
+    ec2 = _resource("ec2", region)  # TODO: take a resource as an argument
+    subnet = ec2.Subnet(subnet_arn)
     try:
-        user.load()
-        return user
-    except botocore.exceptions.ClientError as exc:
-        if exc.response.get("Error", {}).get("Code") == "NoSuchEntity":
+        subnet.load()
+        return subnet
+    except ClientError as e:
+        if e.response.get("Error", {}).get("Code") == "NoSuchEntity":
             return None
         else:
-            raise exc
+            raise e
 
 
 def get_available_regions() -> List[str]:
-    import boto3
-
     try:
         client = boto3.client("ec2")
     except NoRegionError:
@@ -220,6 +233,14 @@ def get_available_regions() -> List[str]:
     return [region["RegionName"] for region in client.describe_regions()["Regions"]]
 
 
+def get_availability_zones(region: str) -> List[str]:
+    client = boto3.client("ec2", region_name=region)
+    return [
+        region["ZoneName"]
+        for region in client.describe_availability_zones()["AvailabilityZones"]
+    ]
+
+
 def launch_gcp_cloud_setup(
     name: str,
     region: str,
@@ -308,7 +329,7 @@ def check_is_feature_flag_on(flag_key: str, default: bool = False) -> bool:
 
 
 def get_active_sessions(
-    project_id: str, session_name: str, api_client: Optional[DefaultApi]
+    project_id: str, session_name: str, api_client: Optional[ProductApi]
 ) -> Any:
     if api_client:
         return api_client.list_sessions_api_v2_sessions_get(
@@ -322,7 +343,7 @@ def get_active_sessions(
         return response["results"]
 
 
-def get_project_directory_name(project_id: str, api_client: DefaultApi = None) -> str:
+def get_project_directory_name(project_id: str, api_client: ProductApi = None) -> str:
     if api_client is None:
         api_client = get_auth_api_client().api_client
 
@@ -334,7 +355,7 @@ def get_project_directory_name(project_id: str, api_client: DefaultApi = None) -
 
 
 def get_working_dir(
-    cluster_config: Dict[str, Any], project_id: str, api_client: DefaultApi = None
+    cluster_config: Dict[str, Any], project_id: str, api_client: ProductApi = None
 ) -> str:
     working_dir: Optional[str] = (
         cluster_config.get("metadata", {}).get("anyscale", {}).get("working_dir")
@@ -412,7 +433,7 @@ def updating_printer() -> Generator[Callable[[str], None], None, None]:
 def wait_for_session_start(
     project_id: str,
     session_name: str,
-    api_client: Optional[DefaultApi] = None,
+    api_client: Optional[ProductApi] = None,
     log: BlockLogger = log,
     block_label: Optional[str] = None,
 ) -> str:
@@ -489,118 +510,6 @@ def wait_for_session_start(
             time.sleep(2)
 
 
-def wait_for_head_node_start(
-    project_id: str,
-    session_name: str,
-    session_id: str,
-    api_client: Optional[DefaultApi] = None,
-) -> str:
-    url = get_endpoint(f"/projects/{project_id}/clusters/{session_id}")
-    log.info(
-        f"Waiting for head node of cluster {session_name} to start. "
-        f"Please view the startup logs at {url}"
-    )
-    if api_client is None:
-        api_client = get_auth_api_client().api_client
-
-    retry_count = 0
-    while True:
-        sessions = api_client.list_sessions_api_v2_sessions_get(
-            project_id=project_id, name=session_name, active_only=False
-        ).results
-
-        if len(sessions) > 0:
-            session = sessions[0]
-
-            if (
-                session.state == "Running" and session.pending_state is None
-            ) or session.state == "AwaitingFileMounts":
-                return cast(str, session.id)
-
-            # Check for start up errors
-            if (
-                session.state_data
-                and session.state_data.startup
-                and session.state_data.startup.startup_error
-            ):
-                raise click.ClickException(
-                    f"Error while starting cluster {session_name}: {session.state_data.startup.startup_error}"
-                )
-            elif session.state and "Errored" in session.state:
-                raise click.ClickException(
-                    f"Error while starting cluster {session_name}: Cluster startup failed due to an error ({session.state})."
-                )
-            elif (
-                retry_count >= 2
-                and session.state
-                and session.state in {"Terminated", "Stopped"}
-                and session.pending_state is None
-            ):
-                # Cluster is created in Terminated state; Ignore the first few state checks.
-                raise click.ClickException(
-                    f"Error while starting cluster {session_name}: Cluster is still in stopped/terminated state."
-                )
-        else:
-            raise click.ClickException(
-                f"Error while starting cluster {session_name}: Cluster doesn't exist."
-            )
-
-        time.sleep(5)
-        retry_count += 1
-
-
-def validate_cluster_configuration(
-    cluster_config_file_name: str,
-    cluster_config: Optional[DefaultApi] = None,
-    api_instance: Optional[DefaultApi] = None,
-) -> None:
-    assert api_instance
-
-    if not os.path.isfile(cluster_config_file_name):
-        raise click.ClickException(
-            "The configuration file {} does not exist. Please provide a valid config file.".format(
-                cluster_config_file_name
-            )
-        )
-
-    if not cluster_config:
-        try:
-            with open(cluster_config_file_name) as f:
-                cluster_config = yaml.safe_load(f)
-        except (ValueError, yaml.YAMLError):
-            raise click.ClickException(
-                "\tThe configuration file {} does not have a valid format. "
-                "\n\tPlease look at https://github.com/ray-project/ray/blob/master/python/ray/autoscaler/aws/example-full.yaml "
-                "for an example configuration file.".format(cluster_config_file_name)
-            )
-
-    try:
-        api_instance.validate_cluster_api_v2_sessions_validate_cluster_post(
-            body={"config": json.dumps(cluster_config)}
-        )
-    except ApiException as e:
-        error = json.loads(json.loads(e.body)["error"]["detail"])
-        path = ".".join(error["path"])
-        if error["path"]:
-            formatted_error = 'Error occured at "{k}: {v}" because {message}.\nSchema description for {k}:\n{schema}'.format(
-                k=path,
-                v=error["instance"],
-                message=error["message"],
-                schema=json.dumps(error["schema"], indent=4, sort_keys=True),
-            )
-        else:
-            formatted_error = 'Error occured at "{v}" because {message}.\nSchema description:\n{schema}'.format(
-                v=error["instance"],
-                message=error["message"],
-                schema=json.dumps(error["schema"], indent=4, sort_keys=True),
-            )
-        raise click.ClickException(
-            "The configuration file {0} is not valid.\n{1}".format(
-                cluster_config_file_name, formatted_error
-            )
-        )
-
-
 def _get_rsync_args(rsync_exclude: List[str], rsync_filter: List[str]) -> List[str]:
     rsync_exclude_args = [["--exclude", exclude] for exclude in rsync_exclude]
     rsync_filter_args = [["--filter", f"dir-merge,- {f}"] for f in rsync_filter]
@@ -891,3 +800,163 @@ def get_current_cluster_id() -> Optional[str]:
 def str_data_size(s: str) -> int:
     """Returns the size of the string when encoded to raw bytes."""
     return len(s.encode("utf-8"))
+
+
+def get_user_env_aws_account(region: str) -> str:
+    """Get the AWS account used in the user environment"""
+    return boto3.client("sts", region_name=region).get_caller_identity()["Account"]
+
+
+def prepare_cloudformation_template(
+    region: str, cfn_stack_name: str, cloud_id: str
+) -> str:
+    with open(f"{anyscale.conf.ROOT_DIR_PATH}/anyscale-cloud-setup.yaml", "r") as f:
+        body = f.read()
+
+    azs = get_availability_zones(region)
+    subnet_templates: List[str] = []
+    subnets_route_table_association: List[str] = []
+    subnets_references: List[str] = []
+
+    vpc_cidr = ipaddress.ip_network(VPC_CIDR_RANGE)
+    if len(azs) > 4:
+        subnet_cidrs = [s for s in vpc_cidr.subnets(prefixlen_diff=3)]
+    else:
+        subnet_cidrs = [s for s in vpc_cidr.subnets(prefixlen_diff=2)]
+
+    for i, az in enumerate(azs):
+        subnet_templates.append(
+            f"""
+  Subnet{i}:
+    Type: AWS::EC2::Subnet
+    Properties:
+        VpcId: !Ref VPC
+        AvailabilityZone: {az}
+        CidrBlock: {subnet_cidrs[i]}
+        MapPublicIpOnLaunch: true
+        Tags:
+        - Key: Name
+          Value: {cfn_stack_name}-subnet-{az}
+        - Key: anyscale-cloud-id
+          Value: {cloud_id}"""
+        )
+
+        subnets_route_table_association.append(
+            f"""
+  Subnet{i}RouteTableAssociation:
+    Type: AWS::EC2::SubnetRouteTableAssociation
+    Properties:
+        RouteTableId: !Ref PublicRouteTable
+        SubnetId: !Ref Subnet{i}"""
+        )
+
+        subnets_references.append(f"!Ref Subnet{i}")
+
+    body = body.replace("$SUBNETS_TEMPLATES", "\n".join(subnet_templates))
+    body = body.replace(
+        "$SUBNETS_ROUTE_TABLE_ASSOCIATION", "\n".join(subnets_route_table_association),
+    )
+    body = body.replace("$SUBNETS_REFERENCES", ",".join(subnets_references))
+    body = body.replace("$ALLOWED_ORIGIN", shared_anyscale_conf.ANYSCALE_HOST)
+
+    return body
+
+
+def get_latest_ray_version():
+    """
+    Gets latest Ray version from PYPI. This method should not
+    assume Ray is already installed.
+    """
+    try:
+        response = requests.get("https://pypi.org/pypi/ray/json")
+        latest_version = response.json()["info"]["version"]
+    except Exception as e:
+        log.debug(
+            f"Unable to get latest Ray version from https://pypi.org/pypi/ray/json {str(e)}"
+        )
+        latest_version = DEFAULT_RAY_VERSION
+    return latest_version
+
+
+def get_ray_and_py_version_for_default_cluster_env() -> Tuple[str, str]:
+    py_version = "".join(str(x) for x in sys.version_info[0:2])
+    if py_version not in ["36", "37", "38"]:
+        raise ValueError(
+            "No default cluster env for python version {}. Please use a version of python between 3.6 and 3.8.".format(
+                py_version
+            )
+        )
+    try:
+        import ray
+
+        ray_version = ray.__version__
+        if version.parse(ray_version) < version.parse(MINIMUM_RAY_VERSION):
+            raise ValueError(
+                f"No default cluster env for Ray version {ray_version}. Please upgrade "
+                f'to a version >= {MINIMUM_RAY_VERSION} with `pip install "ray>={MINIMUM_RAY_VERSION}"`.'
+            )
+        if "dev0" in ray_version:
+            raise ValueError(
+                f"Your locally installed Ray version is {ray_version}. "
+                "There is no default cluster environments for nightly versions of Ray."
+            )
+    except ImportError:
+        # Use latest Ray version if Ray not locally installed, because Anyscale cluster envs for
+        # new Ray versions are available before the open source package is released on PYPI.
+        ray_version = get_latest_ray_version()
+        log.debug(
+            f"Ray is not installed locally. Using latest Ray version {ray_version} for "
+            "the cluster env."
+        )
+
+    return ray_version, py_version
+
+
+def validate_job_config_dict(
+    config_dict: Dict[str, Any], api_client: ProductApi
+) -> None:
+    """
+    Throws an exception if there are invalid values in the config dict.
+    """
+    compute_config: Optional[Union[ComputeTemplate, DecoratedComputeTemplate]] = None
+    if "compute_config" in config_dict and isinstance(
+        config_dict["compute_config"], str
+    ):
+        compute_config = get_cluster_compute_from_name(
+            config_dict["compute_config"], api_client
+        )
+    elif "compute_config_id" in config_dict:
+        cluster_compute_id = config_dict["compute_config_id"]
+        compute_config = api_client.get_compute_template_api_v2_compute_templates_template_id_get(
+            cluster_compute_id
+        ).result
+
+    if compute_config and compute_config.archived_at:
+        raise click.ClickException(
+            "This job is using an archived compute config. To submit this job, specify a new compute config."
+        )
+
+
+def populate_dict_with_workspace_config_if_exists(
+    config_dict: Dict[str, Any], anyscale_api_client: BaseApi
+) -> Dict[str, Any]:
+    """
+    If the job is submitted from the workspace, we populate values from the workspace
+    if the config doesn't specify them, such as cluster env and compute config.
+    """
+    if "ANYSCALE_EXPERIMENTAL_WORKSPACE_ID" in os.environ:
+        if "ANYSCALE_SESSION_ID" in os.environ:
+            cluster = anyscale_api_client.get_cluster(
+                os.environ["ANYSCALE_SESSION_ID"]
+            ).result
+            # If the job configs are not specified, infer them from the workspace:
+            if "build_id" not in config_dict and "cluster_env" not in config_dict:
+                config_dict["build_id"] = cluster.cluster_environment_build_id
+            if "project_id" not in config_dict:
+                config_dict["project_id"] = cluster.project_id
+            if (
+                "compute_config" not in config_dict
+                and "compute_config_id" not in config_dict
+            ):
+                config_dict["compute_config_id"] = cluster.cluster_compute_id
+    return config_dict
