@@ -1,4 +1,4 @@
-from typing import Any, Dict, Optional
+from typing import Any, Dict, Optional, Union
 from unittest.mock import ANY, Mock, patch
 
 import click
@@ -7,10 +7,12 @@ import pytest
 import yaml
 from yaml.loader import SafeLoader
 
+from anyscale.conf import IDLE_TIMEOUT_DEFAULT_MINUTES
 from anyscale.controllers.cluster_compute_controller import (
     ClusterComputeController,
     CreateClusterComputeConfigModel,
 )
+from anyscale.sdk.anyscale_client.models.archive_status import ArchiveStatus
 from anyscale.sdk.anyscale_client.models.cluster_compute import ClusterCompute
 from anyscale.sdk.anyscale_client.models.cluster_computes_query import (
     ClusterComputesQuery,
@@ -21,10 +23,15 @@ from anyscale.sdk.anyscale_client.models.clustercompute_list_response import (
 from anyscale.sdk.anyscale_client.models.clustercompute_response import (
     ClustercomputeResponse,
 )
+from anyscale.sdk.anyscale_client.models.compute_template import ComputeTemplate
+from anyscale.sdk.anyscale_client.models.compute_template_query import (
+    ComputeTemplateQuery,
+)
 from anyscale.sdk.anyscale_client.models.create_cluster_compute import (
     CreateClusterCompute,
 )
 from anyscale.sdk.anyscale_client.models.text_query import TextQuery
+from anyscale.utils.entity_arg_utils import EntityType, IdBasedEntity, NameBasedEntity
 
 
 @pytest.fixture()
@@ -80,6 +87,18 @@ def example_compute_config() -> Dict[str, Any]:
     return yaml.load(example_compute_config_str, Loader=SafeLoader)
 
 
+# Used for generating the compute config read model from the write model.
+# These fields may have these values if the customer didn't specify them,
+# or because of backwards compatibility reasons.
+def compute_config_with_default_fields(
+    write_compute_config: Dict[str, Any]
+) -> Dict[str, Any]:
+    default_fields = dict(
+        idle_termination_minutes=IDLE_TIMEOUT_DEFAULT_MINUTES, region=None,
+    )
+    return dict(**write_compute_config, **default_fields)
+
+
 @pytest.fixture()
 def mock_api_client() -> Mock:
     mock_api_client = Mock()
@@ -135,9 +154,10 @@ def test_create(
     with patch.multiple("yaml", load=mock_load):
         cluster_compute_controller.create(mock_cluster_compute_io, name)
 
-    example_compute_config["region"] = None
     cluster_compute_controller.anyscale_api_client.create_cluster_compute.assert_called_once_with(
-        CreateClusterCompute(name=ANY, config=example_compute_config)
+        CreateClusterCompute(
+            name=ANY, config=compute_config_with_default_fields(example_compute_config)
+        )
     )
 
 
@@ -177,12 +197,50 @@ def test_delete(
         )
 
 
+@pytest.mark.parametrize(
+    "cluster_compute_entity",
+    [IdBasedEntity(id="cpt_123"), NameBasedEntity(name="compute_config_name")],
+)
+def test_archive(
+    mock_auth_api_client,
+    cluster_compute_entity: Union[IdBasedEntity, NameBasedEntity],
+    compute_template_test_data: ComputeTemplate,
+) -> None:
+    cluster_compute_controller = ClusterComputeController()
+
+    if cluster_compute_entity.type is EntityType.NAME:
+        cluster_compute_controller.api_client.search_compute_templates_api_v2_compute_templates_search_post = Mock(
+            return_value=Mock(results=[compute_template_test_data])
+        )
+        cluster_compute_controller.archive(cluster_compute_entity)
+        cluster_compute_controller.api_client.search_compute_templates_api_v2_compute_templates_search_post.assert_called_once_with(
+            ComputeTemplateQuery(
+                orgwide=True,
+                name={"equals": cluster_compute_entity.name},
+                include_anonymous=True,
+                archive_status=ArchiveStatus.ALL,
+            )
+        )
+        cluster_compute_controller.api_client.archive_compute_template_api_v2_compute_templates_compute_template_id_archive_post.assert_called_once_with(
+            compute_template_test_data.id
+        )
+    elif cluster_compute_entity.type is EntityType.ID:
+        cluster_compute_controller.archive(cluster_compute_entity)
+        cluster_compute_controller.anyscale_api_client.get_compute_template.assert_called_once_with(
+            cluster_compute_entity.id
+        )
+        cluster_compute_controller.api_client.archive_compute_template_api_v2_compute_templates_compute_template_id_archive_post.assert_called_once_with(
+            cluster_compute_entity.id
+        )
+
+
 @pytest.mark.parametrize("cluster_compute_name", [None, "test_cluster_compute_name"])
 @pytest.mark.parametrize("cluster_compute_id", [None, "test_cluster_compute_id"])
 def test_get(
     mock_auth_api_client,
     cluster_compute_name: Optional[str],
     cluster_compute_id: Optional[str],
+    compute_template_test_data: ComputeTemplate,
 ):
     cluster_compute_controller = ClusterComputeController()
     if (cluster_compute_name is None and cluster_compute_id is None) or (
@@ -201,6 +259,9 @@ def test_get(
                 return_value=Mock(id=cluster_compute_id)
             ),
         ):
+            cluster_compute_controller.api_client.get_compute_template_api_v2_compute_templates_template_id_get = Mock(
+                return_value=Mock(result=compute_template_test_data)
+            )
             cluster_compute_controller.get(
                 cluster_compute_name=cluster_compute_name,
                 cluster_compute_id=cluster_compute_id,
