@@ -3,14 +3,20 @@ import os
 from pathlib import Path
 import tempfile
 import time
+from types import SimpleNamespace
 from typing import Any, Dict, List, Optional, Tuple, Type
-from unittest.mock import Mock, mock_open, patch
+from unittest.mock import MagicMock, Mock, mock_open, patch
 
 import click
 import pytest
 import yaml
 
-from anyscale.client.openapi_client import CreateProductionJob, ProductionJobConfig
+from anyscale.client.openapi_client.models.create_internal_production_job import (
+    CreateInternalProductionJob,
+)
+from anyscale.client.openapi_client.models.production_job_config import (
+    ProductionJobConfig,
+)
 from anyscale.controllers.job_controller import (
     _working_dir_is_remote_uri,
     JobConfig,
@@ -19,6 +25,7 @@ from anyscale.controllers.job_controller import (
     LogsLogger,
     MiniJobRun,
 )
+from anyscale.sdk.anyscale_client.models.compute_template import ComputeTemplate
 from anyscale.util import PROJECT_NAME_ENV_VAR
 
 
@@ -76,13 +83,45 @@ def patch_jobs_anyscale_api_client(base_mock_anyscale_api_client: Mock):
         yield
 
 
+@pytest.mark.parametrize("workspace_id", [None, "test_workspace_id"])
+def test_generate_config_from_entrypoint(
+    mock_auth_api_client, workspace_id: Optional[str]
+):
+    mock_logger = Mock()
+    job_controller = JobController(log=mock_logger)
+    entrypoint = ["python", "test.py"]
+    name = "test_name"
+    description = "test_description"
+
+    mock_get_default_cluster_compute = Mock(
+        return_value=Mock(id="mock_compute_config_id")
+    )
+
+    mock_validate_successful_build = Mock()
+    mock_get_default_cluster_env_build = Mock(return_value=Mock(id="mock_build_id"))
+    with patch.multiple(
+        "anyscale.controllers.job_controller",
+        get_default_cluster_compute=mock_get_default_cluster_compute,
+        validate_successful_build=mock_validate_successful_build,
+        get_default_cluster_env_build=mock_get_default_cluster_env_build,
+    ):
+        job_config = job_controller.generate_config_from_entrypoint(
+            entrypoint, name, description, workspace_id
+        )
+
+    assert job_config.entrypoint == "python test.py"
+    assert job_config.name == name
+    assert job_config.description == description
+    assert job_config.workspace_id == workspace_id
+
+
 @pytest.mark.parametrize(
     "config_dict",
     [
         {
             "entrypoint": "mock_entrypoint",
             "build_id": "mock_build_id",
-            "compute_config_id": "mock_compute_config_id",
+            "compute_config_id": "cpt_123",
         },
         {
             "entrypoint": "mock_entrypoint",
@@ -111,7 +150,10 @@ def patch_jobs_anyscale_api_client(base_mock_anyscale_api_client: Mock):
 )
 @pytest.mark.parametrize("use_default_project", [True, False])
 def test_submit_job(
-    mock_auth_api_client, config_dict: Dict[str, Any], use_default_project: bool,
+    mock_auth_api_client,
+    config_dict: Dict[str, Any],
+    use_default_project: bool,
+    compute_template_test_data: ComputeTemplate,
 ) -> None:
     config_project_id = config_dict.get("project_id")
     mock_logger = Mock()
@@ -131,15 +173,15 @@ def test_submit_job(
     mock_get_build_from_cluster_env_identifier = Mock(
         return_value=Mock(id="mock_build_id")
     )
-    mock_get_cluster_compute_from_name = Mock(
-        return_value=Mock(id="mock_compute_config_id")
+    job_controller.api_client.get_compute_template_api_v2_compute_templates_template_id_get = Mock(
+        return_value=Mock(result=compute_template_test_data)
     )
-    mock_get_default_cluster_compute = Mock(
-        return_value=Mock(id="mock_compute_config_id")
-    )
-    mock_register_compute_template = Mock(
-        return_value=Mock(id="mock_compute_config_id")
+    job_controller.api_client.search_compute_templates_api_v2_compute_templates_search_post = Mock(
+        return_value=Mock(results=[compute_template_test_data])
     )
+    mock_get_cluster_compute_from_name = Mock(return_value=compute_template_test_data)
+    mock_get_default_cluster_compute = Mock(return_value=compute_template_test_data)
+    mock_register_compute_template = Mock(return_value=compute_template_test_data)
 
     mock_validate_successful_build = Mock()
     mock_get_default_cluster_env_build = Mock(return_value=Mock(id="mock_build_id"))
@@ -173,15 +215,16 @@ def test_submit_job(
     else:
         final_project_id = "mock_project_id"
     job_controller.api_client.create_job_api_v2_decorated_ha_jobs_create_post.assert_called_once_with(
-        CreateProductionJob(
+        CreateInternalProductionJob(
             name="mock_name",
             description="mock_description",
             project_id=final_project_id,
+            workspace_id=None,
             config=ProductionJobConfig(
                 **{
                     "entrypoint": "mock_entrypoint",
                     "build_id": "mock_build_id",
-                    "compute_config_id": "mock_compute_config_id",
+                    "compute_config_id": compute_template_test_data.id,
                 }
             ),
         )
@@ -192,7 +235,7 @@ def test_submit_job(
     if "compute_config" in config_dict:
         compute_config = config_dict["compute_config"]
         if isinstance(compute_config, str):
-            mock_get_cluster_compute_from_name.assert_called_once_with(compute_config)
+            mock_get_cluster_compute_from_name.assert_called_with(compute_config)
         elif isinstance(compute_config, dict):
             mock_register_compute_template.assert_called_once_with(compute_config)
     elif "cloud" in config_dict:
@@ -887,7 +930,7 @@ def test_job_submit_parse_logic(mock_auth_api_client) -> None:
         job_controller.generate_config_from_file.return_value = mock_config
         job_controller.submit("file", entrypoint=[], is_entrypoint_cmd=False)
         job_controller.generate_config_from_file.assert_called_once_with(
-            "file", name=None, description=None
+            "file", name=None, description=None, workspace_id="fake_workspace_id"
         )
         job_controller.submit_from_config.assert_called_once_with(mock_config)
         job_controller.generate_config_from_file.reset_mock()
@@ -897,80 +940,88 @@ def test_job_submit_parse_logic(mock_auth_api_client) -> None:
         job_controller.generate_config_from_entrypoint.return_value = mock_config
         job_controller.submit("file", entrypoint=["entrypoint"], is_entrypoint_cmd=True)
         job_controller.generate_config_from_entrypoint.assert_called_once_with(
-            ["file", "entrypoint"], None, None
+            ["file", "entrypoint"], None, None, "fake_workspace_id"
         )
         job_controller.submit_from_config.assert_called_once_with(mock_config)
 
 
-def test_job_schedule_parse_logic(mock_auth_api_client) -> None:
+def test_job_wait(mock_auth_api_client) -> None:
     job_controller = JobController()
-    job_controller.generate_config_from_entrypoint = Mock()  # type: ignore
-    job_controller.generate_config_from_file = Mock()  # type: ignore
-    job_controller.schedule_from_config = Mock()  # type: ignore
+    job_controller.api_client = Mock()
+    job_controller._resolve_job_object = Mock(return_value=SimpleNamespace(id="id"))  # type: ignore
+    spin_mock = MagicMock()
+    job_controller.log = MagicMock()
+    job_controller.log.spinner.return_value.__enter__.return_value = spin_mock
 
-    # We are not in a workspace, so entrypoint should not be allowed
-    with pytest.raises(click.ClickException):
-        job_controller.schedule(
-            "file",
-            entrypoint=["entrypoint"],
-            is_entrypoint_cmd=False,
-            cron_expression="0 0 1 * *",
-        )
+    def get_job_state(states_to_return, mock_state=Mock()):
+        mock_state.i = 0
+
+        def g(*args, **kwargs):
+            nonlocal mock_state
+            state = states_to_return[mock_state.i]
+            mock_state.i += 1
+            mock_state.state = state
+            return state
+
+        return g, mock_state
+
+    # Test the happy path
+    states_to_return = ["PENDING", "AWAITING_CLUSTER_START", "SUCCESS"]
+    job_controller._get_job_state, mock_state = get_job_state(states_to_return)  # type: ignore
+
+    assert job_controller.wait("name")
+    assert mock_state.i == 3
+    assert mock_state.state == "SUCCESS"
+    spin_mock.succeed.assert_called_once()
+
+    # Test the failure case
+    spin_mock.reset_mock()
+    states_to_return = [
+        "PENDING",
+        "AWAITING_CLUSTER_START",
+        "ERRORED",
+        "OUT_OF_RETRIES",
+    ]
+    job_controller._get_job_state, mock_state = get_job_state(states_to_return)  # type: ignore
 
     with pytest.raises(click.ClickException):
-        job_controller.schedule(
-            "file",
-            entrypoint=["entrypoint"],
-            is_entrypoint_cmd=True,
-            cron_expression="0 0 1 * *",
-        )
+        job_controller.wait("name")
+    assert mock_state.i == 4
+    assert mock_state.state == "OUT_OF_RETRIES"
+    spin_mock.fail.assert_called_once()
+
+    # Test the non default case
+    spin_mock.reset_mock()
+    states_to_return = [
+        "PENDING",
+        "AWAITING_CLUSTER_START",
+        "ERRORED",
+        "SHOULD_NOT_REACH",
+        "SHOULD_NOT_REACH",
+        "SHOULD_NOT_REACH",
+    ]
+    job_controller._get_job_state, mock_state = get_job_state(states_to_return)  # type: ignore
+
+    assert job_controller.wait("name", target_state="ERRORED")
+    assert mock_state.state == "ERRORED"
+    assert mock_state.i == 3
+    spin_mock.succeed.assert_called_once()
+
+    # Test timeout logic
+    spin_mock.reset_mock()
+    states_to_return = [
+        "PENDING",
+        "AWAITING_CLUSTER_START",
+        "ERRORED",
+        "SHOULD_NOT_REACH",
+        "SHOULD_NOT_REACH",
+        "SHOULD_NOT_REACH",
+    ]
+    job_controller._get_job_state, mock_state = get_job_state(states_to_return)  # type: ignore
 
     with pytest.raises(click.ClickException):
-        job_controller.schedule(
-            "file",
-            entrypoint=["entrypoint", "commands"],
-            is_entrypoint_cmd=True,
-            cron_expression="0 0 1 * *",
-        )
+        job_controller.wait("name", timeout_secs=0.00001)
 
-    # Simulate a workspace
-    with patch.dict(
-        os.environ, {"ANYSCALE_EXPERIMENTAL_WORKSPACE_ID": "fake_workspace_id"}
-    ):
-        # Fails due to is_entrypoint_cmd being False
-        with pytest.raises(click.ClickException):
-            job_controller.schedule(
-                "file",
-                entrypoint=["entrypoint"],
-                is_entrypoint_cmd=False,
-                cron_expression="0 0 1 * *",
-            )
-
-        mock_config = Mock()
-        job_controller.generate_config_from_file.return_value = mock_config
-        job_controller.schedule(
-            "file", entrypoint=[], is_entrypoint_cmd=False, cron_expression="0 0 1 * *"
-        )
-        job_controller.generate_config_from_file.assert_called_once_with(
-            "file", name=None, description=None
-        )
-        job_controller.schedule_from_config.assert_called_once_with(
-            mock_config, cron_expression="0 0 1 * *"
-        )
-        job_controller.generate_config_from_file.reset_mock()
-        job_controller.schedule_from_config.reset_mock()
-
-        mock_config = Mock()
-        job_controller.generate_config_from_entrypoint.return_value = mock_config
-        job_controller.schedule(
-            "file",
-            entrypoint=["entrypoint"],
-            is_entrypoint_cmd=True,
-            cron_expression="0 0 1 * *",
-        )
-        job_controller.generate_config_from_entrypoint.assert_called_once_with(
-            ["file", "entrypoint"], None, None
-        )
-        job_controller.schedule_from_config.assert_called_once_with(
-            mock_config, cron_expression="0 0 1 * *"
-        )
+    # Test parsing logic
+    with pytest.raises(click.ClickException):
+        job_controller.wait("name", target_state="FAKE_STATE")
