@@ -13,7 +13,6 @@ import ray
 import requests
 import yaml
 
-import anyscale
 from anyscale.client.openapi_client import SessionListResponse
 from anyscale.client.openapi_client.models.app_config import AppConfig
 from anyscale.client.openapi_client.models.build import Build
@@ -27,7 +26,6 @@ from anyscale.utils.connect_helpers import (
     AnyscaleClientConnectResponse,
     AnyscaleClientContext,
 )
-import anyscale.utils.runtime_env
 
 
 RAY_VERSION = "1.6.0"
@@ -84,6 +82,7 @@ def _make_build() -> Build:
         created_at=datetime.now(),
         last_modified_at=datetime.now(),
         docker_image_name="docker_image_name",
+        is_byod=False,
     )
 
 
@@ -209,7 +208,6 @@ def _make_test_builder(
     ray.__commit__ = RAY_COMMIT
     ray.__version__ = RAY_VERSION
     ray.util.client.ray.is_connected.return_value = False
-    anyscale.utils.runtime_env.runtime_env_setup = Mock()
 
     def disconnected(*a: Any, **kw: Any) -> None:
         ray.util.client.ray.is_connected.return_value = False
