@@ -3,7 +3,10 @@ from unittest.mock import Mock
 import pytest
 
 from anyscale.utils.ray_version_checker import check_required_ray_version
-from frontend.cli.tests.test_connect import RAY_COMMIT, RAY_VERSION
+
+
+RAY_VERSION = "1.6.0"
+RAY_COMMIT = "7916500c43de46721c51e6b95fb51cfa2c6078ba"
 
 
 @pytest.mark.parametrize(
